#region Copyright (c) 2001-2009 Infragistics, Inc. All Rights Reserved
/* ---------------------------------------------------------------------*
*                           Infragistics, Inc.                          *
*              Copyright (c) 2001-2009 All Rights reserved               *
*                                                                       *
*                                                                       *
* This file and its contents are protected by United States and         *
* International copyright laws.  Unauthorized reproduction and/or       *
* distribution of all or any portion of the code contained herein       *
* is strictly prohibited and will result in severe civil and criminal   *
* penalties.  Any violations of this copyright will be prosecuted       *
* to the fullest extent possible under law.                             *
*                                                                       *
* THE SOURCE CODE CONTAINED HEREIN AND IN RELATED FILES IS PROVIDED     *
* TO THE REGISTERED DEVELOPER FOR THE PURPOSES OF EDUCATION AND         *
* TROUBLESHOOTING. UNDER NO CIRCUMSTANCES MAY ANY PORTION OF THE SOURCE *
* CODE BE DISTRIBUTED, DISCLOSED OR OTHERWISE MADE AVAILABLE TO ANY     *
* THIRD PARTY WITHOUT THE EXPRESS WRITTEN CONSENT OF INFRAGISTICS, INC. *
*                                                                       *
* UNDER NO CIRCUMSTANCES MAY THE SOURCE CODE BE USED IN WHOLE OR IN     *
* PART, AS THE BASIS FOR CREATING A PRODUCT THAT PROVIDES THE SAME, OR  *
* SUBSTANTIALLY THE SAME, FUNCTIONALITY AS ANY INFRAGISTICS PRODUCT.    *
*                                                                       *
* THE REGISTERED DEVELOPER ACKNOWLEDGES THAT THIS SOURCE CODE           *
* CONTAINS VALUABLE AND PROPRIETARY TRADE SECRETS OF INFRAGISTICS,      *
* INC.  THE REGISTERED DEVELOPER AGREES TO EXPEND EVERY EFFORT TO       *
* INSURE ITS CONFIDENTIALITY.                                           *
*                                                                       *
* THE END USER LICENSE AGREEMENT (EULA) ACCOMPANYING THE PRODUCT        *
* PERMITS THE REGISTERED DEVELOPER TO REDISTRIBUTE THE PRODUCT IN       *
* EXECUTABLE FORM ONLY IN SUPPORT OF APPLICATIONS WRITTEN USING         *
* THE PRODUCT.  IT DOES NOT PROVIDE ANY RIGHTS REGARDING THE            *
* SOURCE CODE CONTAINED HEREIN.                                         *
*                                                                       *
* THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.              *
* --------------------------------------------------------------------- *
*/
#endregion Copyright (c) 2001-2009 Infragistics, Inc. All Rights Reserved

using System;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Globalization;
using System.Drawing.Design;

using Infragistics.Shared;

namespace Infragistics.Win.UltraWinSchedule
{
	/// <summary>
	/// Base class for scheduler data binding classes.
	/// </summary>
	public abstract class DataBindingsBase: SubObjectBase
	{
		private object dataSource = null;
		private string dataMember = null;
		private Control bindingContextControl = null;
		private BindingContext bindingContext = null;
		private BindingManagerBase bindingManager = null;
		private string[] propertyNames = null;
		private PropertyDescriptor[] propertyDescriptors = null;
		private UltraCalendarInfo ownerCalendarInfo = null;
		private bool showErrorMessageBox = true;

		
		private bool isSynchronized = false;

		//	BF 5.18.04	UWS940
		private bool isInOnBindingListChanged = false;

		//	BF 11/9/06	BR16579
		private bool isInSyncListObjects = false;

		/// <summary>
		/// Constructor.
		/// </summary>
        /// <param name="ownerCalendarInfo">The owning UltraCalendarInfo</param>
		/// <param name="size">The size</param>
		protected DataBindingsBase(UltraCalendarInfo ownerCalendarInfo, int size)
		{
			this.ownerCalendarInfo = ownerCalendarInfo;

			this.propertyNames = new string[size];
			this.propertyDescriptors = new PropertyDescriptor[size];
		}

		internal string[] PropertyNames
		{
			get { return this.propertyNames; }
		}

		internal PropertyDescriptor[] PropertyDescriptors
		{
			get { return this.propertyDescriptors; }
		}

		internal UltraCalendarInfo OwnerCalendarInfo
		{
			get { return this.ownerCalendarInfo; }
		}

		internal bool ShowErrorMessageBox
		{
			get { return this.showErrorMessageBox; }
			set { this.showErrorMessageBox = value; }
		}

		/// <summary>
		/// Gets/sets the source of the data for binding.
		/// </summary>
		[LocalizedDescription("LD_DataBindingsBase_P_DataSource")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[RefreshProperties(RefreshProperties.All)]
        [AttributeProvider(typeof(IListSource))]
		public object DataSource
		{
			get
			{
				return this.dataSource;
			}
			set
			{
				if(this.dataSource!=null && this.dataSource.Equals(value))
					return;

				if(this.dataSource!=null && this.dataSource is IComponent)
					((IComponent)this.dataSource).Disposed -= new EventHandler( this.OnDataSourceDisposed );

				this.dataSource = value;

				this.SetDataBindingHelper();

				if(this.dataSource!=null && this.dataSource is IComponent)
					((IComponent)this.dataSource).Disposed += new EventHandler( this.OnDataSourceDisposed );

				// Notify listeners
				this.NotifyPropChange( PropertyIds.DataSource );
			}
		}

		private void OnDataSourceDisposed( object sender, EventArgs e ) 
		{
			this.SetDataBinding(null, null);
		}

		/// <summary>
		/// Gets/sets a sub-list of the DataSource to bind to.
		/// </summary>
		[LocalizedDescription("LD_DataBindingsBase_P_DataMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor("System.Windows.Forms.Design.DataMemberListEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing")]
		[RefreshProperties(RefreshProperties.All)]  
		public string DataMember
		{
			get
			{
				return this.dataMember;
			}
			set
			{
				if(this.dataMember!=null && this.dataMember.Equals(value))
					return;
				
				if(value==null)
					value = string.Empty;

				this.dataMember = value;

				this.SetDataBindingHelper();
			
				// Notify listeners
				this.NotifyPropChange( PropertyIds.DataMember );
			}
		}

		/// <summary>
		/// Gets control used for binding context.
		/// </summary>
		[LocalizedDescription("LD_DataBindingsBase_P_BindingContextControl")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[TypeConverter("System.ComponentModel.ReferenceConverter")]
		[RefreshProperties(RefreshProperties.All)]  
		public Control BindingContextControl
		{
			get
			{
				return this.bindingContextControl;
			}
			set
			{
				// AS 5/19/04
				//if(this.bindingContextControl!=null && this.bindingContextControl.Equals(value))
				if(this.bindingContextControl == value)
					return;

				// AS 5/19/04
				// If we had a binding context control, null out the
				// binding context.
				//
				if (this.bindingContextControl != null)
					this.bindingContext = null;

				this.bindingContextControl = value;

				// AS 5/19/04
				// If we have a binding context control, get the
				// new binding context.
				//
				if (this.bindingContextControl != null)
					this.bindingContext = this.bindingContextControl.BindingContext;

				this.SetDataBindingHelper();

				// Notify listeners
				this.NotifyPropChange( PropertyIds.BindingContextControl );
			}
		}

		/// <summary>
		/// Gets/sets binding context.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public BindingContext BindingContext
		{
			get
			{
				return this.bindingContext;
			}
			set
			{
				// AS 5/19/04
				// If the value was null and null is passed in, don't do anything
				//
				//if(this.bindingContext!=null && this.bindingContext.Equals(value))
				// MD 1/28/09 - TFS13144
				// This check was incorrect. We should bail when the passed in value is equal.
				//if(this.bindingContext != value)
				if ( this.bindingContext == value )
					return;

				this.bindingContext = value;

				// AS 5/19/04
				// I spoke with Joe about this and if we had a
				// binding context and they're setting the BindingContext
				// then we should null out the binding context control.
				this.bindingContextControl = null;

				this.SetDataBindingHelper();

				// Notify listeners
				this.NotifyPropChange( PropertyIds.BindingContext );
			}
		}

		/// <summary>
		/// Sets the DataSource and DataMember properties in one atomic operation.
		/// </summary>
		/// <param name="dataSource">Indicates the source of data.</param>
		/// <param name="dataMember">Indicates a sub-list of the DataSource.</param>
		public void SetDataBinding(object dataSource, string dataMember)
		{
			this.dataSource = dataSource;
			this.dataMember = dataMember;

			this.SetDataBindingHelper();
		}

		private void SetDataBindingHelper()
		{
			BindingManagerBase oldBindingManager = this.bindingManager;

			// Remove the events from the old datasource (important for GC).
			WireDataSource(false);

			if (this.dataSource!=null && this.bindingContext!=null) 
				this.bindingManager = this.bindingContext[this.dataSource, this.dataMember];
			else 
				this.bindingManager = null;

			//	BF 10/26/06	BR17064
			if ( this.dataSource == null && this.bindingManager == null )
				this.OnDataSourceRemoved();

			this.FixSingleChapteredColumn();

			// Add the events on the new datasource.
			WireDataSource(true);

			if(this.bindingManager != oldBindingManager)
			{
				this.InitializePropertyDescriptors();

				// Every time binding list changes we will show message box for first data error which is not 
				// handled with binding error events. 
				this.showErrorMessageBox = true;

				
				

				this.isSynchronized = false;

				//	BF 5.18.04	UWS979
				//
				//	Calling BeginInvoke on the form when in InitializeComponent
				//	causes an exception to be thrown, since the form's handle
				//	has not yet been created.
				//
				//if(this.bindingContextControl!=null)
				if ( this.bindingContextControl!=null &&
					 this.bindingContextControl.IsHandleCreated )
					this.bindingContextControl.BeginInvoke(new MethodInvoker(this.SyncCallback));
			}
		}

		internal void SyncCallback()
		{
			if(!this.isSynchronized)
				this.SyncListObjects();
		}

		internal void WireDataSource(bool addEvents) 
		{
			if(this.bindingManager!=null) 
			{
				CurrencyManager cm = this.bindingManager as CurrencyManager;
				if(cm!=null)
				{
					IBindingList bl = cm.List as IBindingList;
					if(bl!=null)
					{
						// ListChanged
						ListChangedEventHandler tmLCEH = new ListChangedEventHandler( this.OnBindingListChanged );
						if(addEvents)
							bl.ListChanged += tmLCEH;
						else
							bl.ListChanged -= tmLCEH;
					}
				}
			}
		}

		private void OnBindingListChanged(object sender, ListChangedEventArgs e)
		{
			try
			{
				//	BF 5.18.04	UWS979
				this.isInOnBindingListChanged = true;

				// Every time binding list changes we will show message box for first data error which is not 
				// handled with binding error events. 
				this.showErrorMessageBox = true;

				switch(e.ListChangedType)
				{
					case ListChangedType.PropertyDescriptorAdded:
					case ListChangedType.PropertyDescriptorChanged:
					case ListChangedType.PropertyDescriptorDeleted:
						this.InitializePropertyDescriptors();
						this.ItemList.DirtyList(true);
						break;

					case ListChangedType.ItemChanged:

						// If specific item was changed..
						if(e.NewIndex >= 0)
						{
							// ..find it..
							foreach(IDataBoundItem item in this.ItemList)
								if(item.ListIndex == e.NewIndex)
								{
									// ..dirty that item..
									item.DirtyItem(false);

									// ..and return.
									return;
								}
						}

						// In any other case, do the synchronization.
						goto case ListChangedType.Reset;

					case ListChangedType.ItemMoved:

						// If specific item was moved..
						if(e.NewIndex >= 0 && e.OldIndex >= 0 && e.NewIndex != e.OldIndex)
						{
                            // ..find it..
                            int currNewIndex = -1;
                            foreach (IDataBoundItem item in this.ItemList)
                            {
                                currNewIndex = -1;
                                if (item.ListIndex == e.OldIndex)
                                {
                                    // ..update..
                                    currNewIndex = e.NewIndex;
                                }
                                else if (e.NewIndex > e.OldIndex) //item was moved from an earlier position to a later one
                                {
                                    if (item.ListIndex > e.NewIndex)
                                    {
                                        return;
                                    }
                                    else if (item.ListIndex > e.OldIndex && item.ListIndex <= e.NewIndex)
                                    {
                                        currNewIndex = item.ListIndex - 1;
                                    }
                                }
                                else //item was moved from a later position to an earlier one
                                {
                                    if (item.ListIndex >= e.NewIndex && item.ListIndex < e.OldIndex)
                                    {
                                        currNewIndex = item.ListIndex + 1;
                                    }
                                }

                                if (currNewIndex >= 0)
                                {
                                    item.ListIndex = currNewIndex;
                                    item.ListObject = this.CurrencyManagerList[currNewIndex];
                                }
                            }
						}

						// In any other case, do the synchronization.
						goto case ListChangedType.Reset;

					case ListChangedType.ItemAdded:

						// If specific item was added..
						if(e.NewIndex >= 0)
						{
							bool isValidItem;

							
							IDataBoundItem newItem = this.DataBoundItemCreator( e.NewIndex, 
								this.CurrencyManagerList[e.NewIndex], out isValidItem );

							if(isValidItem)
							{
								// ..update..
								foreach(IDataBoundItem item in this.ItemList)
								{
									if(item.ListIndex >= e.NewIndex)
									{
										item.ListIndex++;
										item.ListObject = this.CurrencyManagerList[item.ListIndex];
									}
								}

								// ..add new item..

								this.ItemList.Add(newItem);

								this.ItemList.DirtyList(false);
							}

							// ..and return.
							return;
						}

						// In any other case, do the synchronization.
						goto case ListChangedType.Reset;

					case ListChangedType.ItemDeleted:

						// If specific item was deleted..
						if(e.NewIndex >= 0)
						{
							// ..update..
							foreach(IDataBoundItem item in this.ItemList)
							{
								if(item.ListIndex > e.NewIndex)
								{
									item.ListIndex--;
									item.ListObject = this.CurrencyManagerList[item.ListIndex];
								}
							}

							// ..find it..
							for(int i=0; i<this.ItemList.Count; i++)
							{
								IDataBoundItem item = (IDataBoundItem) this.ItemList[i];

								if(item.ListIndex == e.NewIndex)
								{
									// ..delete..
									this.ItemList.RemoveAt(i);

									this.ItemList.DirtyList(false);

									// ..and return.
									return;
								}
							}
						}

						// In any other case, do the synchronization.
						goto case ListChangedType.Reset;

					case ListChangedType.Reset:

						this.SyncListObjects();
						break;
				}
			}
			finally
			{
				//	BF 5.18.04	UWS979
				this.isInOnBindingListChanged = false;
			}
		}

		private void FixSingleChapteredColumn()
		{
			if(this.bindingManager==null)
				return;

			bool allChapteredColumns = true;
			string firstChapteredPropName = null;

			// Loop over all the props to make sure there is at least
			// one no chaptered property. This can happen if the DataMember
			// is not specified. In that case the propertys can contain
			// one or more chaptered properties only.
			foreach(PropertyDescriptor prop in this.bindingManager.GetItemProperties())
			{
				// check if this property is chaptered
				if ( typeof(IList).IsAssignableFrom( prop.PropertyType ) && 
					!typeof(Array).IsAssignableFrom( prop.PropertyType ) )
				{
					// keep track of the name of the first chaptered column
					if(firstChapteredPropName==null)
						firstChapteredPropName = prop.Name;
				}
				else
				{
					allChapteredColumns = false;
					break;
				}
			}

			if(allChapteredColumns)
			{
				// Use the first chaptered property name.
				if(firstChapteredPropName!=null)
				{
					string tmDataMember = this.dataMember;

					// Append the name to the original datamember (if specified).
					if(tmDataMember!=null && tmDataMember.Length!=0) 
						tmDataMember = tmDataMember + "." + firstChapteredPropName;
					else
						tmDataMember = firstChapteredPropName;

					// Get the child bands binding manager.
					this.bindingManager = this.BindingContext[this.dataSource, tmDataMember];
				}
			}
		}

		/// <summary>
		/// Caches property descriptors for all relevant members.
		/// </summary>
		protected void InitializePropertyDescriptors()
		{
			int i;

			Debug.Assert(this.propertyNames.Length==this.propertyDescriptors.Length);

			for(i=0; i<propertyDescriptors.Length; i++)
				this.propertyDescriptors[i] = null;

			if(this.bindingManager==null)
				return;

			PropertyDescriptorCollection props = DataBindingUtils.GetItemProperties( this.bindingManager, 
				new DataBindingUtils.ValuePropertyDescriptorCreator(this.ItemPropertyDescriptorCreator) );

			foreach(PropertyDescriptor prop in props)
			{
				// Check if this property is chaptered..
				if ( typeof(IList).IsAssignableFrom( prop.PropertyType )   && 
					!typeof(Array).IsAssignableFrom( prop.PropertyType ) )
					// ..skip if it is.
					continue;

				for(i=0; i<propertyNames.Length; i++)
				{
					if(propertyNames[i]!=null && propertyNames[i].Length>0 && this.propertyDescriptors[i]==null &&
						0 == string.Compare(propertyNames[i], prop.Name, false, CultureInfo.CurrentCulture ) )
					{
						this.propertyDescriptors[i] = prop;
						break;
					}
				}
			}

			
			for(i=0; i<propertyNames.Length; i++)
			{
				if(propertyNames[i]!=null && propertyNames[i].Length>0 && this.propertyDescriptors[i]==null)
					throw new Exception(SR.GetString("LE_DataBinding_BadMemberName") + propertyNames[i]);
			}
		}

		/// <summary>
		/// Tries to perform synchronization between items and data source.
		/// </summary>
		protected virtual void SyncListObjects()
		{
			if(this.bindingManager==null || this.ownerCalendarInfo.DesignMode)
				return;

			//	BF 11/9/06	BR16579
			#region Obsolete code
			//	this.isSynchronized = true;
			//
			//	this.FullSynchronization(this.ItemList, this.CurrencyManagerList);
			//
			//	this.ItemList.DirtyList(false);
			#endregion Obsolete code

			//	BF 11/9/06	BR16579
			//	Wrapped this in a try/finally and added an "in progress" flag
			try
			{
				this.isInSyncListObjects = true;

				this.isSynchronized = true;

				this.FullSynchronization(this.ItemList, this.CurrencyManagerList);

				this.ItemList.DirtyList(false);
			}
			finally
			{
				this.isInSyncListObjects = false;
			}
		}

		//	BF 11/9/06	BR16579
		/// <summary>
		/// Returns whether the 'SyncListObjects' method is currently in progress,
		/// which implies that the Appointments/Owners/Notes collection is being
		/// populated from a data source.
		/// </summary>
		internal bool IsSynchronizingWithDataSource
		{
			get{ return this.isInSyncListObjects; }
		}

		/// <summary>
		/// Performs full synchronization between items and data source.
		/// </summary>
		/// <param name="itemList">The item list.</param>
		/// <param name="boundList">The bound list.</param>
		protected virtual void FullSynchronization(IList itemList, IList boundList)
		{
			int i;

			// Create hash table for fast lookup.
			Hashtable hash = new Hashtable( boundList.Count, 0.90f );
			
			// Fill the hash table.
			for(i=0; i<boundList.Count; i++)
			{
				object comparisionListObject = DataBindingUtils.GetObjectForComparison( boundList[i] );

				if(comparisionListObject==null)
					// If we can't get comparison object, we can't synchronize.
					return;

				hash[comparisionListObject] = i;
			}

			// Create array which will be filled with reused items.
			object[] reusedItemList = new object[boundList.Count];

            //  BF 2/20/08  BR29761
            //
            //  The process that follows attempts to reuse existing instances,
            //  which causes a problem for Owners because they have keys. Since
            //  the Owners collection is going to be cleared immediately after
            //  we iterate this list, we can get rid of the keys now so that
            //  we don't cause unnecessary key validation errors. The validation
            //  will then happen at the very end of this routine, when the recycled
            //  instances are added to the Owners collection.
            //
            //  Note that for Appointments/Notes, this is all irrelevant, and the
            //  ClearKeys method does nothing accordingly.
            //
            this.ClearKeys();

			// Find items which can be reused (they are the same but realocated).

            //  BF 11/28/07 BR28635
            #region Obsolete code
            //for(i=0; i<itemList.Count; i++)
            //{
            //    IDataBoundItem item = (IDataBoundItem)itemList[i];

            //    object boundData = null;
            //    if(item!=null && item.ListObject!=null)
            //        boundData = hash[ DataBindingUtils.GetObjectForComparison( item.ListObject ) ];

            //    if(boundData!=null)
            //    {
            //        int boundDataIndex = (int)boundData;

            //        item.ListIndex = boundDataIndex;
            //        item.ListObject = boundList[boundDataIndex];
            //        reusedItemList[boundDataIndex] = item.CloneItem();

            //        //~ ZS 4/28/04 UWS898 - Need to dirty the item as we changed binding list index and object. 
            //        item.DirtyItem(false); 
            //    }
            //}
            #endregion Obsolete code

            //  BF 11/28/07 BR28635
            //  Use a copy of the specified IList, since the specified IList
            //  is a reference to the actual Appointments or Owners collection,
            //  the contents of which could change during the iteration.
            int itemCount = itemList.Count;
            ArrayList itemListCopy = new ArrayList( itemCount );
            for ( i = 0; i < itemCount; i ++ )
            { 
                itemListCopy.Add( itemList[i] );
            }

			for( i = 0; i < itemCount; i ++ )
			{
				IDataBoundItem item = (IDataBoundItem)itemListCopy[i];

				object boundData = null;
				if( item != null && item.ListObject != null )
					boundData = hash[ DataBindingUtils.GetObjectForComparison( item.ListObject ) ];

				if( boundData != null )
				{
					int boundDataIndex = (int)boundData;

					item.ListIndex = boundDataIndex;
					item.ListObject = boundList[boundDataIndex];
					
                    //  ZS 4/28/04 UWS898 - Need to dirty the item as we changed binding list index and object. 

                    //  BF 11/28/07 BR28635
                    //
                    //  Dirtying the item before cloning it accomplishes absolutely nothing,
                    //  since the clone will not pick up the changes made by dirtying it, and
                    //  the item itself will be removed from the collection (and thus disposed of)
                    //  immediately after this loop is done iterating. I moved this up from below
                    //  the following line.
                    //
					item.DirtyItem(false); 
					
                    reusedItemList[boundDataIndex] = item.CloneItem();
				}
			}

			// Clear original array of items.
			itemList.Clear();
			
			// Fill original array with..
			for(i=0; i<boundList.Count; i++)
			{
				// ..old but realocated items, 
				IDataBoundItem item = (IDataBoundItem)reusedItemList[i];

				if(item==null)
				{
					bool isValidItem;

					

					// ..or with new items which correspond to bound data.
					item = this.DataBoundItemCreator(i, boundList[i], out isValidItem);

					if(!isValidItem)
						continue;
				}
				//	BF 5/1/07	BR22201
				//	If we reuse an item, we have to clear the bitmask that tracks
				//	which properties are in synch with the data source, or the recycled
				//	instance will have stale property values.
				else
					item.DirtyPropertyCache();


				itemList.Add(item);
			}
		}

		/// <summary>
		/// List associated with the currency manager. 		
		/// </summary>
		protected IList CurrencyManagerList
		{
			get
			{
				if(this.bindingManager is CurrencyManager)
					return ((CurrencyManager)this.bindingManager).List;
				
				return null;
			}
		}

		/// <summary>
		/// Binding list. 		
		/// </summary>
		internal IBindingList BindingList
		{
			get
			{
				return this.CurrencyManagerList as IBindingList;
			}
		}

		/// <summary>
		/// Synchronizes data between data source and component.
		/// </summary>
		public void RefreshData()
		{
			
			this.SyncListObjects();
		}

		internal object ConvertHelper(object source, Type destinationType)
		{
			bool validConversion;

			return DataBindingUtils.DataBindConvert(source, destinationType, CultureInfo.InvariantCulture, 
				null, null, out validConversion);
		}

		internal static void BeginEdit(object listObject)
		{
			IEditableObject editableObject = listObject as IEditableObject;
			if(null != editableObject)
				editableObject.BeginEdit();
		}

		internal static void EndEdit(object listObject)
		{
			IEditableObject editableObject = listObject as IEditableObject;
			if(null != editableObject)
				editableObject.EndEdit();
		}

		#region Abstract Methods

		/// <summary>
		/// Creator for class-specific property descriptor.
		/// </summary>
		/// <param name="type">The type</param>
		/// <param name="name">The name</param>
		/// <returns></returns>
		protected abstract PropertyDescriptor ItemPropertyDescriptorCreator(Type type, string name);

		/// <summary>
		/// Underlying list of items.
		/// </summary>
		internal abstract DataBoundItemsBase ItemList { get; }

		
		internal abstract IDataBoundItem DataBoundItemCreator(int listIndex, object listObject, out bool isValidItem);

		#endregion

        //  BF 1/24/08  BR29935
        //  Made this virtual

		/// <summary>
		/// Indicates whether the <see cref="DataBindingsBase"/> object needs to be persisted.
		/// </summary>
        /// <returns>true if this object needs to be serialized; otherwise, false</returns>
        public virtual bool ShouldSerialize()
		{
			if(this.dataSource!=null || this.dataMember!=null || this.bindingContextControl!=null)
				return true;
			else
				return false;
		}

		//	BF 5.18.04	UWS979
		internal bool IsInOnBindingListChanged
		{
			get{ return this.isInOnBindingListChanged; }
		}

		//	BF 6.17.04	NAS2004 Vol3 - Notes Data Binding
		/// <summary>
		/// Resets all properties of the <see cref="DataBindingsBase"/> object to their default values
		/// </summary>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public virtual void Reset()
		{
			this.BindingContext = null;
			this.BindingContextControl = null;
			this.DataMember = null;
			this.DataSource = null;
			this.ResetTag();
		}

		//	BF 10/26/06	BR17064
		#region OnDataSourceRemoved
		/// <summary>
		/// Called when the data source has been removed. Should be overridden in
		/// derived classes to perform any necessary cleanup (i.e., clear collections)
		/// </summary>
		internal protected abstract void OnDataSourceRemoved();
		#endregion OnDataSourceRemoved

        //  BF 2/20/08  BR29761
        #region ClearKeys
        internal virtual void ClearKeys(){}
        #endregion ClearKeys
	}

	internal interface IDataBoundItem
	{
		object ListObject { get; set; }
		int ListIndex { get; set; }
		
		bool DirtyItem(bool checkValidity);
		object CloneItem();

		//	BF 5/1/07	BR22201
		void DirtyPropertyCache();
	}

	internal abstract class DataBoundItemsBase : IList
	{
		public virtual void DirtyList(bool dirtyEveryItem)
		{
			throw new NotImplementedException();
		}

		#region IList Members

		public virtual bool IsReadOnly
		{
			get { throw new NotImplementedException(); }
		}

		public virtual object this[int index]
		{
			get { throw new NotImplementedException(); }
			set { throw new NotImplementedException(); }
		}

		public virtual void RemoveAt(int index) { throw new NotImplementedException(); }

		public virtual void Insert(int index, object value) { throw new NotImplementedException(); }

		public virtual void Remove(object value) { throw new NotImplementedException(); }

		public virtual bool Contains(object value) { throw new NotImplementedException(); }

		public virtual void Clear() { throw new NotImplementedException(); }

        /// <summary>
        /// Returns the index of the object in the collection
        /// </summary>
        /// <param name="value">The object whose presence should be checked in the collection.</param>
        /// <returns>The index of the object in the collection.</returns>
        public virtual int IndexOf(object value) { throw new NotImplementedException(); }

		public virtual int Add(object value) { throw new NotImplementedException(); }

		public virtual bool IsFixedSize { get { throw new NotImplementedException(); } }

		#endregion

		#region ICollection Members

		public virtual bool IsSynchronized
		{
			get { throw new NotImplementedException(); }
		}

		public virtual int Count
		{
			get { throw new NotImplementedException(); }
		}

		public virtual void CopyTo(Array array, int index) { throw new NotImplementedException(); }

		public virtual object SyncRoot 
		{ 
			get { throw new NotImplementedException(); } 
		}

		#endregion

		#region IEnumerable Members

        /// <summary>
        /// IEnumerable Interface Implementation returns a type safe enumerator
        /// </summary>		
        /// <returns>A type safe enumerator.</returns>		
        public virtual IEnumerator GetEnumerator() { throw new NotImplementedException(); }

		#endregion
	}

    //  BF 1/24/08  BR29935
    //
    //  I commented out both the AppointmentsDataBinding and NotesDataBinding classes
    //  in their entirety, since the changes (most of it just cleanup) were too numerous
    //  to tag individually. I moved whatever commonality there was to a base class (see
    //  ActivitiesDataBindingBase).
    //
    #region Obsolete code
    //    #region NotesDataBinding class
    //    /// <summary>
    //    /// Class that handles data binding for the <see cref="UltraCalendarInfo"/> component's <see cref="UltraCalendarInfo.Notes"/> collection.
    //    /// </summary>
    //    /// <remarks>
    //    /// <p class="body">
    //    /// The <b>NAS2004 Volume 3</b> release of UltraWinSchedule includes support for data binding the UltraCalendarInfo component's Notes collection.
    //    /// The Notes collection can be populated from the contents of any IList implementor, for example, a DataTable. To identify the names of the members
    //    /// that contain the property values for each Note object, the <see cref="NotesDataBinding.DateMember"/> and the <see cref="NotesDataBinding.DescriptionMember"/>
    //    /// properties must be set. The contents of the fields that correspond to these members supply the values for the Note object's <see cref="Note.Date"/> and
    //    /// <see cref="Note.Description"/> properties, respectively. Each item in the list (for example, the individual rows in a DataTable) represents a member of
    //    /// the Notes collection. As property values of the Note object change, the new value is written to the corresponding member of the data source.
    //    /// </p>
    //    /// </remarks>
    //    /// <seealso cref="UltraCalendarInfo.DataBindingsForNotes"/>
    //    /// <seealso cref="UltraCalendarInfo.Notes"/>
    //    /// <seealso cref="Note"/>
    //    /// <seealso cref="NotesCollection"/>
    //    /// <seealso cref="AppointmentsDataBinding"/>
    //    /// <seealso cref="OwnersDataBinding"/>
    //    [TypeConverter( typeof( ExpandableObjectConverter ) )]
    //    public class NotesDataBinding: DataBindingsBase
    //    {
    //        #region Member variables

    //        private string dateTimeFormatString = null;
    //        private IFormatProvider dateTimeFormatInfo = null;
    //        private NotesAdapter notesAdapter = null;
    //        private DateSerializationMode allPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
    //        private DateSerializationMode dateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;

    //        private static PropertyIds[] notificationMapping = 
    //            new PropertyIds[] 
    //            { 
    //                PropertyIds.AllPropertiesMember, 
    //                PropertyIds.DateMember, 
    //                PropertyIds.DescriptionMember, 
    //                PropertyIds.OwnerKeyMember, 
    //                PropertyIds.DataKeyMember 
    //            };

    //        #endregion Member variables

    //        #region Constructor
    //        /// <summary>
    //        /// (Internal) Creates a new instance of the <see cref="NotesDataBinding"/> class.
    //        /// </summary>
    //        /// <param name="ownerCalendarInfo">The <see cref="UltraCalendarInfo"/> component whose <see cref="UltraCalendarInfo.Notes"/> collection is to be bound.</param>
    //        internal NotesDataBinding(UltraCalendarInfo ownerCalendarInfo): 
    //                 base(ownerCalendarInfo, Enum.GetNames( typeof(NotesDataBindingPropertyId) ).Length - 4)
    //        {
    //            this.notesAdapter = new NotesAdapter( this.OwnerCalendarInfo.Notes );
    //        }
    //        #endregion Constructor

    //        #region NotesAdapter class (Private)
    //#if DEBUG
    //        /// <summary>
    //        /// IList implementor that serves as a liason between
    //        /// the Notes collection and the data source.
    //        /// </summary>
    //#endif
    //        private class NotesAdapter: DataBoundItemsBase
    //        {
    //            #region Member variables

    //            private NotesCollection notes;
    			
    //            #endregion Member variables

    //            #region Constructor
    //            /// <summary>
    //            /// Creates a new instance of the NotesAdapter class.
    //            /// </summary>
    //            /// <param name="notes">A reference to the Notes collection for which this instance provides data.</param>
    //            public NotesAdapter( NotesCollection notes )
    //            {
    //                this.notes = notes;
    //            }
    //            #endregion Constructor

    //            #region DirtyList
    //            /// <summary>
    //            /// Overridden; called in response to the ListChanged event.
    //            /// </summary>
    //            /// <param name="dirtyEveryItem">Boolean specifying whether every member of the Notes collection is to be dirtied.</param>
    //            public override void DirtyList( bool dirtyEveryItem )
    //            {
    //                if( dirtyEveryItem )
    //                    this.notes.DirtyAllNotes();

    //                //	Dirty version number, so that any subset collections are rebuilt
    //                this.notes.CollectionChanged();

    //                //	Issue a property change notification
    //                this.notes.NotifyPropChange(PropertyIds.Note);
    //            }
    //            #endregion DirtyList

    //            #region Clear
    //            /// <summary>
    //            /// Overridden; called when the Notes collection is to be cleared.
    //            /// </summary>
    //            public override void Clear()
    //            {
    //                this.notes.ClearHelper(false);
    //            }
    //            #endregion Clear

    //            #region Add
    //            /// <summary>
    //            /// Overridden; called when a new member is to be added to the Notes collection.
    //            /// </summary>
    //            public override int Add( object value )
    //            {
    //                //return this.notes.AddNoteHelper( value as Note, false );
    //                return this.notes.AddNoteHelper( value as Note );
    //            }
    //            #endregion Add

    //            #region Count
    //            /// <summary>
    //            /// Overridden; returns the count of the Notes collection
    //            /// </summary>
    //            public override int Count
    //            { 
    //                get { return this.notes.Count; } 
    //            }
    //            #endregion Count

    //            #region RemoveAt
    //            /// <summary>
    //            /// Overridden; removes the member of the Notes collection whose
    //            /// ordinal position corresponds to the specified index.
    //            /// </summary>
    //            public override void RemoveAt( int index )
    //            {
    //                //	BF 5/31/06	BR12852
    //                //this.notes.RemoveHelper( this.notes[index] );
    //                this.notes.RemoveHelper( this.notes[index], false );
    //            }
    //            #endregion RemoveAt

    //            #region Indexer ('this' operator)
    //            /// <summary>
    //            /// Overridden; returns the member of the Notes collection whose
    //            /// ordinal position corresponds to the specified index.
    //            /// </summary>
    //            public override object this[int index]
    //            {
    //                get { return this.notes[index]; }
    //            }
    //            #endregion Indexer ('this' operator)

    //            #region GetEnumerator
    //            /// <summary>
    //            /// IEnumerable Interface Implementation returns a type safe enumerator
    //            /// </summary>		
    //            /// <returns>A type safe enumerator.</returns>		
    //            public override IEnumerator GetEnumerator()
    //            {
    //                return this.notes.GetEnumerator();
    //            }
    //            #endregion GetEnumerator
    //        }
    //        #endregion NotesAdapter class (Private)

    //        #region ItemList
    //        /// <summary>
    //        /// Returns the NotesAdapter instance associated with this NotesDataBinding instance.
    //        /// </summary>
    //        internal override DataBoundItemsBase ItemList 
    //        { 
    //            get
    //            {
    //                return this.notesAdapter;
    //            }
    //        }
    //        #endregion ItemList

    //        #region ItemPropertyDescriptorCreator
    //        /// <summary>
    //        /// Creator for class-specific property descriptor.
    //        /// </summary>
    //        /// <param name="type">The type</param>
    //        /// <param name="name">The name</param>
    //        protected override PropertyDescriptor ItemPropertyDescriptorCreator( Type type, string name )
    //        {
    //            throw new Exception( SR.GetString("LE_AppointmentsDataBinding_CannotBind") );
    //        }
    //        #endregion ItemPropertyDescriptorCreator

    //        #region DataBoundItemCreator
    //        /// <summary>
    //        /// Creats a new instance of the Note object
    //        /// </summary>
    //        /// <param name="listIndex">The value for the BindingListIndex property of the Note to be created.</param>
    //        /// <param name="listObject">The value for the BindingListObject property of the Note to be created.</param>
    //        /// <param name="isValidItem">On return, specifies whether the Note was successfully created.</param>
    //        /// <returns></returns>
    //        internal override IDataBoundItem DataBoundItemCreator( int listIndex, object listObject, out bool isValidItem )
    //        {
    //            Note note = new Note( this.OwnerCalendarInfo, listIndex, listObject );

    //            isValidItem = ((IDataBoundItem)note).DirtyItem( true );

    //            note.SubObjectPropChanged += new SubObjectPropChangeEventHandler( note.OnThisObjectChanged );

    //            return ((IDataBoundItem)note);
    //        }
    //        #endregion DataBoundItemCreator

    //        #region ConvertDateTimeHelper
    //        /// <summary>
    //        /// Converts the specified source object to the specified destinationType.
    //        /// </summary>
    //        /// <param name="source">The object to convert</param>
    //        /// <param name="destinationType">The type to which to convert the source object.</param>
    //        /// <returns>The converted object</returns>
    //        internal object ConvertDateTimeHelper( object source, Type destinationType )
    //        {
    //            bool validConversion;

    //            return DataBindingUtils.DataBindConvert(source, destinationType, CultureInfo.InvariantCulture, 
    //                this.dateTimeFormatString, this.dateTimeFormatInfo, out validConversion);
    //        }
    //        #endregion ConvertDateTimeHelper

    //        #region GetPropertyName
    //        /// <summary>
    //        /// Returns the property name that corresponds to the specified NotesDataBindingPropertyId.
    //        /// </summary>
    //        /// <param name="currentPropertyId">The NotesDataBindingPropertyId whose property name is to be returned.</param>
    //        private string GetPropertyName( NotesDataBindingPropertyId currentPropertyId )
    //        {
    //            return this.PropertyNames[(int)currentPropertyId];
    //        }
    //        #endregion GetPropertyName

    //        #region SetPropertyName
    //        /// <summary>
    //        /// Sets the property name that corresponds to the specified NotesDataBindingPropertyId.
    //        /// </summary>
    //        /// <param name="name">The property name to be returned.</param>
    //        /// <param name="currentPropertyId">The NotesDataBindingPropertyId whose property name is to be returned.</param>
    //        private void SetPropertyName( string name, NotesDataBindingPropertyId currentPropertyId )
    //        {
    //            string newValue = DataBindingUtils.StripMember(name);

    //            if( newValue != this.PropertyNames[(int)currentPropertyId] )
    //            {
    //                //	BF 11/1/05	BR07410
    //                string oldValue = this.PropertyNames[(int)currentPropertyId];
    //                bool dirtyAll = (oldValue != null && oldValue.Length > 0);

    //                this.PropertyNames[(int)currentPropertyId] = newValue;

    //                this.InitializePropertyDescriptors();

    //                //	BF 8/3/05	BR05397
    //                //	BF 11/1/05	BR07410
    //                //if ( this.OwnerCalendarInfo != null )
    //                if ( dirtyAll && this.OwnerCalendarInfo != null )
    //                {
    //                    NotesCollection notes = this.OwnerCalendarInfo.Notes;
    //                    for ( int i = 0; i < notes.Count; i ++ )
    //                    {
    //                        Note note = notes[i];
    //                        note.DirtyBoundPropertyValue( currentPropertyId );
    //                    }
    //                }

    //                this.NotifyPropChange( NotesDataBinding.notificationMapping[ (int)currentPropertyId ] );
    //            }		
    //        }
    //        #endregion SetPropertyName

    //        #region Public Properties
    			
    //            #region DateMember
    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.Date"/> property when data bound.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_DateMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string DateMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( NotesDataBindingPropertyId.DateMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, NotesDataBindingPropertyId.DateMember );
    //            }
    //        }	
    //            #endregion DateMember

    //            #region DescriptionMember
    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.Description"/> property when data bound.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_DescriptionMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string DescriptionMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( NotesDataBindingPropertyId.DescriptionMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, NotesDataBindingPropertyId.DescriptionMember );
    //            }
    //        }	
    //            #endregion DescriptionMember

    //            #region OwnerKeyMember
    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.OwnerKey"/> property when data bound.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_OwnerKeyMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string OwnerKeyMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( NotesDataBindingPropertyId.OwnerKeyMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, NotesDataBindingPropertyId.OwnerKeyMember );
    //            }
    //        }	
    //            #endregion OwnerKeyMember

    //            #region DataKeyMember
    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.DataKey"/> property when data bound.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_DataKeyMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string DataKeyMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( NotesDataBindingPropertyId.DataKeyMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, NotesDataBindingPropertyId.DataKeyMember );
    //            }
    //        }	
    //            #endregion DataKeyMember

    //            #region AllPropertiesMember
    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the <see cref="Note"/> from serialized data.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_AllPropertiesMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string AllPropertiesMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( NotesDataBindingPropertyId.AllPropertiesMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, NotesDataBindingPropertyId.AllPropertiesMember );
    //            }
    //        }	
    //            #endregion AllPropertiesMember

    //            #region DateTimeFormatString
    //        /// <summary>
    //        /// Gets/sets the format string used for DateTime serialization.
    //        /// </summary>
    //        /// <seealso cref="NotesDataBinding.DateTimeFormatInfo"/>
    //        [LocalizedDescription("NotesDataBinding_P_DateTimeFormatString")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        public string DateTimeFormatString
    //        {
    //            get
    //            {
    //                return this.dateTimeFormatString;
    //            }
    //            set
    //            {
    //                if( value != this.dateTimeFormatString )
    //                {
    //                    this.dateTimeFormatString = value;

    //                    this.NotifyPropChange( PropertyIds.DateTimeFormatString );
    //                }
    //            }
    //        }	
    //            #endregion DateTimeFormatString

    //            #region DateTimeFormatInfo
    //        /// <summary>
    //        /// Gets/sets the IFormatProvider implementor used for DateTime serialization.
    //        /// </summary>
    //        /// <seealso cref="NotesDataBinding.DateTimeFormatString"/>
    //        [Editor(typeof(FormatProviderEditor), typeof(UITypeEditor))]
    //        [DesignerSerializationVisibility( DesignerSerializationVisibility.Visible )]
    //        [TypeConverter(typeof( FormatProviderTypeConverter ))]
    //        [LocalizedDescription("LD_DataBinding_P_FormatProvider")]
    //        [LocalizedCategory("LC_Behavior")]
    //        public IFormatProvider DateTimeFormatInfo
    //        {
    //            get
    //            {
    //                return this.dateTimeFormatInfo;
    //            }
    //            set
    //            {
    //                if( value != this.dateTimeFormatInfo )
    //                {
    //                    this.dateTimeFormatInfo = value;

    //                    this.NotifyPropChange( PropertyIds.DateTimeFormatInfo );
    //                }
    //            }
    //        }

    //        /// <summary>
    //        /// Indicates whether the <see cref="NotesDataBinding.DateTimeFormatInfo"/> property needs to be serialized.
    //        /// </summary>
    //        /// <returns>True when the contents of the <see cref="NotesDataBinding.DateTimeFormatInfo"/> property needs to be serialized.</returns>
    //        /// <seealso cref="NotesDataBinding.DateTimeFormatInfo"/>
    //        protected bool ShouldSerializeDateTimeFormatInfo() 
    //        {
    //            return this.dateTimeFormatInfo != null;
    //        }
     
    //        /// <summary>
    //        /// Resets the <see cref="NotesDataBinding.DateTimeFormatInfo"/> property to its default value.
    //        /// </summary>
    //        /// <seealso cref="NotesDataBinding.DateTimeFormatInfo"/>
    //        [EditorBrowsable(EditorBrowsableState.Advanced)]
    //        public void ResetDateTimeFormatInfo() 
    //        {
    //            this.DateTimeFormatInfo = null;
    //        }
    //            #endregion DateTimeFormatInfo

    //            #region AllPropertiesMemberSerializationMode
    //        /// <summary>
    //        /// Gets/sets the date serialization mode for the <see cref="NotesDataBinding.AllPropertiesMember"/>.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_AllPropertiesMemberSerializationMode")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(DateSerializationMode.SerializeDatesAsUniversalTime)]
    //        public DateSerializationMode AllPropertiesMemberSerializationMode
    //        {
    //            get { return this.allPropertiesMemberSerializationMode; }
    //            set
    //            { 
    //                this.allPropertiesMemberSerializationMode = value; 

    //                this.NotifyPropChange( PropertyIds.AllPropertiesMember );
    //            }
    //        }
    //            #endregion AllPropertiesMemberSerializationMode

    //            #region DateMembersSerializationMode
    //        /// <summary>
    //        /// Date serialization mode for the <see cref="Note.Date"/> property.
    //        /// </summary>
    //        [LocalizedDescription("NotesDataBinding_P_DateMembersSerializationMode")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(DateSerializationMode.SerializeDatesAsLocalTime)]
    //        public DateSerializationMode DateMembersSerializationMode
    //        {
    //            get { return this.dateMembersSerializationMode; }
    //            set
    //            { 
    //                this.dateMembersSerializationMode = value;

    //                this.NotifyPropChange( PropertyIds.DateMembersSerializationMode );
    //            }
    //        }
    //            #endregion DateMembersSerializationMode

    //        #endregion Public Properties

    //        #region Public Methods

    //            #region ShouldSerialize
    //        /// <summary>
    //        /// Returns whether the <see cref="NotesDataBinding"/> object needs to be serialized.
    //        /// </summary>
    //        /// <returns>true if this object needs to be serialized; otherwise, false</returns>
    //        public new bool ShouldSerialize()
    //        {
    //            return	this.DateMember != null ||
    //                    this.DescriptionMember != null ||
    //                    this.OwnerKeyMember != null ||
    //                    this.AllPropertiesMember != null ||
    //                    this.DateTimeFormatString != null ||
    //                    this.allPropertiesMemberSerializationMode!=DateSerializationMode.SerializeDatesAsUniversalTime || 
    //                    this.dateMembersSerializationMode!=DateSerializationMode.SerializeDatesAsLocalTime || 
    //                    base.ShouldSerialize();										
    //        }
    //            #endregion ShouldSerialize

    //            #region Reset
    //        /// <summary>
    //        /// Resets all properties of the <see cref="NotesDataBinding"/> object to their default values
    //        /// </summary>
    //        [EditorBrowsable(EditorBrowsableState.Advanced)]
    //        public override void Reset()
    //        {
    //            this.AllPropertiesMember = string.Empty;
    //            this.AllPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
    //            this.DataKeyMember = string.Empty;
    //            this.DateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;
    //            this.DateTimeFormatInfo = null;
    //            this.DateTimeFormatString = null;
    //            this.DescriptionMember = string.Empty;
    //            this.OwnerKeyMember = string.Empty;
    			
    //            base.Reset();
    //        }

    //            #endregion Reset

    //        //	BF 10/26/06	BR17064
    //            #region OnDataSourceRemoved
    //        /// <summary>
    //        /// Called when the data source has been removed.
    //        /// </summary>
    //        internal protected override void OnDataSourceRemoved()
    //        {
    //            UltraCalendarInfo calendarInfo = this.OwnerCalendarInfo;
    //            if ( calendarInfo != null )
    //                calendarInfo.Notes.ClearHelper( false );
    //        }
    //            #endregion OnDataSourceRemoved

    //        #endregion	Public Methods

    //    }
    //    #endregion NotesDataBinding class

    //    #region AppointmentsDataBinding class
    //    /// <summary>
    //    /// Scheduler data bindings for appointments.
    //    /// </summary>
    //    [TypeConverter( typeof( ExpandableObjectConverter ) )]
    //    public class AppointmentsDataBinding: DataBindingsBase
    //    {
    //        private string dateTimeFormatString = null;
    //        private IFormatProvider dateTimeFormatInfo = null;
    //        private AppointmentsAdapter appointmentsAdapter = null;

    //        //~ ZS 4/29/04 UWS899 - We deleted old SerializeAsUniversalTime and added this fields instead.
    //        private DateSerializationMode allPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
    //        private DateSerializationMode dateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;

    //        private static PropertyIds[] notificationMapping = 
    //            new PropertyIds[] 
    //            { 
    //                PropertyIds.StartDateTimeMember, 
    //                PropertyIds.EndDateTimeMember, 
    //                PropertyIds.SubjectMember, 
    //                PropertyIds.DescriptionMember, 
    //                PropertyIds.AllDayEventMember, 
    //                PropertyIds.ReminderEnabledMember, 
    //                PropertyIds.ReminderIntervalMember, 
    //                PropertyIds.ReminderUnitsMember, 
    //                PropertyIds.OwnerKeyMember,
    //                PropertyIds.DataKeyMember, 
    //                PropertyIds.RecurrenceMember, 
    //                PropertyIds.RecurrenceIdMember, 
    //                PropertyIds.OriginalStartDateTimeMember, 
    //                PropertyIds.AllPropertiesMember 
    //            };

    //        internal AppointmentsDataBinding(UltraCalendarInfo ownerCalendarInfo): 
    //            base(ownerCalendarInfo, Enum.GetNames( typeof(AppointmentsDataBindingPropertyId) ).Length - 4)
    //        {
    //            this.appointmentsAdapter = new AppointmentsAdapter( this.OwnerCalendarInfo.Appointments );
    //        }

    //        private class AppointmentsAdapter: DataBoundItemsBase
    //        {
    //            private AppointmentsCollection appointments;

    //            public AppointmentsAdapter(AppointmentsCollection appointments)
    //            {
    //                this.appointments = appointments;
    //            }

    //            public override void DirtyList(bool dirtyEveryItem)
    //            {
    //                if(dirtyEveryItem)
    //                    this.appointments.DirtyAllAppointments(false);

    //                /* ZS 4/28/04 UWS898 - Need to dirty appointments collection (maybe number of items was 0). */
    //                this.appointments.BumpVersionNumber();

    //                this.appointments.NotifyPropChange(PropertyIds.Appointment);
    //            }

    //            public override void Clear()
    //            {
    //                this.appointments.ClearHelper(false);
    //            }

    //            public override int Add(object value)
    //            {
    //                return this.appointments.AddAppointmentHelper(value as Appointment, false);
    //            }

    //            public override int Count
    //            { 
    //                get { return this.appointments.Count; } 
    //            }

    //            public override void RemoveAt(int index)
    //            {
    //                this.appointments.RemoveHelper( this.appointments[index], false );
    //            }

    //            public override object this[int index]
    //            {
    //                get { return this.appointments[index]; }
    //            }

    //            /// <summary>
    //            /// IEnumerable Interface Implementation returns a type safe enumerator
    //            /// </summary>		
    //            /// <returns>A type safe enumerator.</returns>		
    //            public override IEnumerator GetEnumerator()
    //            {
    //                return this.appointments.GetEnumerator();
    //            }
    //        }

    //        /// <summary>
    //        /// Underlying list of items.
    //        /// </summary>
    //        internal override DataBoundItemsBase ItemList 
    //        { 
    //            get
    //            {
    //                return this.appointmentsAdapter;
    //            }
    //        }

    //        /// <summary>
    //        /// Creator for class-specific property descriptor.
    //        /// </summary>
    //        /// <param name="type">The type</param>
    //        /// <param name="name">The name</param>
    //        protected override PropertyDescriptor ItemPropertyDescriptorCreator(Type type, string name)
    //        {
    //            throw new Exception( SR.GetString("LE_AppointmentsDataBinding_CannotBind") );
    //        }

    //        internal override IDataBoundItem DataBoundItemCreator(int listIndex, object listObject, out bool isValidItem)
    //        {
    //            Appointment appt = new Appointment(this.OwnerCalendarInfo, listIndex, listObject);

    //            isValidItem = ((IDataBoundItem)appt).DirtyItem(true);

    //            //	BF 3/18/05	BR02914
    //            //	This is no longer needed
    //            //appt.SubObjectPropChanged += new SubObjectPropChangeEventHandler( appt.OnThisObjectChanged );

    //            return ((IDataBoundItem)appt);
    //        }

    //        internal object ConvertDateTimeHelper(object source, Type destinationType)
    //        {
    //            bool validConversion;

    //            return DataBindingUtils.DataBindConvert(source, destinationType, CultureInfo.InvariantCulture, 
    //                this.dateTimeFormatString, this.dateTimeFormatInfo, out validConversion);
    //        }

    //        private string GetPropertyName(AppointmentsDataBindingPropertyId currentPropertyId)
    //        {
    //            return this.PropertyNames[(int)currentPropertyId];
    //        }

    //        private void SetPropertyName(string name, AppointmentsDataBindingPropertyId currentPropertyId)
    //        {
    //            string newValue = DataBindingUtils.StripMember(name);

    //            if(newValue != this.PropertyNames[(int)currentPropertyId])
    //            {
    //                //	BF 11/1/05	BR07410
    //                string oldValue = this.PropertyNames[(int)currentPropertyId];
    //                bool dirtyAll = (oldValue != null && oldValue.Length > 0);

    //                this.PropertyNames[(int)currentPropertyId] = newValue;

    //                this.InitializePropertyDescriptors();

    //                //	BF 8/3/05	BR05397
    //                //
    //                //	In the BR05397 scenario, the SubjectMember field was changed during
    //                //	the lifetime of the application, so that a different value could be
    //                //	displayed. We already had the plumbing in place to refresh the property
    //                //	descriptor, so all that needed to be done was to mark the property dirty
    //                //	for each appointment in the collection.
    //                //
    //                //	BF 11/1/05	BR07410
    //                //~	I really don't understand why this causes BR07410, but it does.
    //                //	Since we didn't used to do this prior to the fix for BR05397, this
    //                //	shouldn't cause any other problems, and it doesn't break BR05397
    //                //	because if the ***Member property wasn't previously set the Appointment
    //                //	property value cannot possibly be stale.
    //                //if ( this.OwnerCalendarInfo != null )
    //                if ( dirtyAll && this.OwnerCalendarInfo != null )
    //                {
    //                    AppointmentsCollection appointments = this.OwnerCalendarInfo.Appointments;
    //                    for ( int i = 0; i < appointments.Count; i ++ )
    //                    {
    //                        Appointment appointment = appointments[i];
    //                        appointment.DirtyBoundPropertyValue( currentPropertyId );
    //                    }
    //                }

    //                this.NotifyPropChange( AppointmentsDataBinding.notificationMapping[ (int) currentPropertyId ] );
    //            }		
    //        }

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s start date when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_StartDateTimeMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string StartDateTimeMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.StartDateTimeMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.StartDateTimeMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s end date when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_EndDateTimeMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string EndDateTimeMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.EndDateTimeMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.EndDateTimeMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s subject property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_SubjectMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string SubjectMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.SubjectMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.SubjectMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s description property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_DescriptionMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string DescriptionMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.DescriptionMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.DescriptionMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s AllDayEvent property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_AllDayEventMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string AllDayEventMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.AllDayEventMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.AllDayEventMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s reminder when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_ReminderEnabledMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string ReminderEnabledMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.ReminderEnabledMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.ReminderEnabledMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the reminder�s DisplayInterval property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_ReminderIntervalMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string ReminderIntervalMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.ReminderIntervalMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.ReminderIntervalMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the reminder�s DisplayIntervalUnits property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_ReminderUnitsMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string ReminderUnitsMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.ReminderUnitsMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.ReminderUnitsMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s owner property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_OwnerKeyMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string OwnerKeyMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.OwnerKeyMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.OwnerKeyMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s DataKey property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_DataKeyMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string DataKeyMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.DataKeyMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.DataKeyMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s Recurrence property to new AppointmentRecurrence. Appointment variances don�t make use of this as they only contain reference to �root� recurrence.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_RecurrenceMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string RecurrenceMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.RecurrenceMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.RecurrenceMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s Recurrence property to existing AppointmentRecurrence in Root Appointment.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_RecurrenceIdMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string RecurrenceIdMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.RecurrenceIdMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.RecurrenceIdMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment�s OriginalStartDateTime property when bound.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_OriginalStartDateTimeMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string OriginalStartDateTimeMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.OriginalStartDateTimeMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.OriginalStartDateTimeMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Gets/sets the field to use to initialize the appointment from serialized data.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_AllPropertiesMember")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        [Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
    //        public string AllPropertiesMember
    //        {
    //            get
    //            {
    //                return this.GetPropertyName( AppointmentsDataBindingPropertyId.AllPropertiesMember );
    //            }
    //            set
    //            {
    //                this.SetPropertyName( value, AppointmentsDataBindingPropertyId.AllPropertiesMember );
    //            }
    //        }	

    //        /// <summary>
    //        /// Specifies format string used for DateTime serialization.
    //        /// </summary>
    //        /// <seealso cref="AppointmentsDataBinding.DateTimeFormatInfo"/>
    //        [LocalizedDescription("LD_AppointmentsDataBinding_P_DateTimeFormatString")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(null)]
    //        public string DateTimeFormatString
    //        {
    //            get
    //            {
    //                return this.dateTimeFormatString;
    //            }
    //            set
    //            {
    //                if(value != this.dateTimeFormatString)
    //                {
    //                    this.dateTimeFormatString = value;

    //                    this.NotifyPropChange( PropertyIds.DateTimeFormatString );
    //                }
    //            }
    //        }	

    //        //	BF 6.17.04	NAS2004 Vol3 - Notes Data Binding
    //        //	Made the DateTimeFormatInfo proeprty design-time visible
    //        #region DateTimeFormatInfo
    //        /// <summary>
    //        /// Gets/sets the IFormatProvider implementor used for DateTime serialization.
    //        /// </summary>
    //        /// <seealso cref="AppointmentsDataBinding.DateTimeFormatString"/>
    //        [Editor(typeof(FormatProviderEditor), typeof(UITypeEditor))]
    //        [DesignerSerializationVisibility( DesignerSerializationVisibility.Visible )]
    //        [TypeConverter(typeof( FormatProviderTypeConverter ))]
    //        [LocalizedDescription("LD_DataBinding_P_FormatProvider")]
    //        [LocalizedCategory("LC_Behavior")]
    //        public IFormatProvider DateTimeFormatInfo
    //        {
    //            get
    //            {
    //                return this.dateTimeFormatInfo;
    //            }
    //            set
    //            {
    //                if(value != this.dateTimeFormatInfo)
    //                {
    //                    this.dateTimeFormatInfo = value;

    //                    this.NotifyPropChange( PropertyIds.DateTimeFormatInfo );
    //                }
    //            }
    //        }	

    //        /// <summary>
    //        /// Indicates whether the <see cref="AppointmentsDataBinding.DateTimeFormatInfo"/> property needs to be serialized.
    //        /// </summary>
    //        /// <returns>True when the contents of the <see cref="AppointmentsDataBinding.DateTimeFormatInfo"/> property needs to be serialized.</returns>
    //        /// <seealso cref="AppointmentsDataBinding.DateTimeFormatInfo"/>
    //        protected bool ShouldSerializeDateTimeFormatInfo() 
    //        {
    //            return this.dateTimeFormatInfo != null;
    //        }
     
    //        /// <summary>
    //        /// Resets the <see cref="AppointmentsDataBinding.DateTimeFormatInfo"/> property to its default value.
    //        /// </summary>
    //        /// <seealso cref="AppointmentsDataBinding.DateTimeFormatInfo"/>
    //        [EditorBrowsable(EditorBrowsableState.Advanced)]
    //        public void ResetDateTimeFormatInfo() 
    //        {
    //            this.DateTimeFormatInfo = null;
    //        }
    //        #endregion DateTimeFormatInfo

    //        /// <summary>
    //        /// Indicates whether the <see cref="AppointmentsDataBinding"/> object needs to be persisted.
    //        /// </summary>
    //        /// <returns>true if this object needs to be serialized; otherwise, false</returns>
    //        public new bool ShouldSerialize()
    //        {
    //            if(this.StartDateTimeMember!=null || this.EndDateTimeMember!=null || this.SubjectMember!=null ||
    //                this.DescriptionMember!=null || this.AllDayEventMember!=null || this.ReminderEnabledMember!=null ||
    //                this.ReminderIntervalMember!=null || this.ReminderUnitsMember!=null || this.OwnerKeyMember!=null ||
    //                this.DataKeyMember!=null || this.RecurrenceMember!=null || this.RecurrenceIdMember!=null ||
    //                this.OriginalStartDateTimeMember!=null || this.AllPropertiesMember!=null || 
    //                this.DateTimeFormatString!=null || 
    //                this.allPropertiesMemberSerializationMode!=DateSerializationMode.SerializeDatesAsUniversalTime || 
    //                this.dateMembersSerializationMode!=DateSerializationMode.SerializeDatesAsLocalTime || 
    //                base.ShouldSerialize())
    //                return true;
    //            else
    //                return false;
    //        }

    //        //~ ZS 4/29/04 UWS899 - We deleted old SerializeAsUniversalTime and added this fields instead.

    //        /// <summary>
    //        /// Date serialization mode for <see cref="AppointmentsDataBinding.AllPropertiesMember"/>. Also used 
    //        /// for <see cref="AppointmentsDataBinding.RecurrenceMember"/>.
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsAdapter_P_AllPropertiesMemberSerializationMode")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(DateSerializationMode.SerializeDatesAsUniversalTime)]
    //        public DateSerializationMode AllPropertiesMemberSerializationMode
    //        {
    //            get { return this.allPropertiesMemberSerializationMode; }
    //            set { this.allPropertiesMemberSerializationMode = value; }
    //        }

    //        /// <summary>
    //        /// Date serialization mode for all date members (<see cref="AppointmentsDataBinding.StartDateTimeMember"/>,
    //        /// <see cref="AppointmentsDataBinding.EndDateTimeMember"/> and <see cref="AppointmentsDataBinding.OriginalStartDateTimeMember"/>).
    //        /// </summary>
    //        [LocalizedDescription("LD_AppointmentsAdapter_P_DateMembersSerializationMode")]
    //        [LocalizedCategory("LC_Data")]
    //        [DefaultValue(DateSerializationMode.SerializeDatesAsLocalTime)]
    //        public DateSerializationMode DateMembersSerializationMode
    //        {
    //            get { return this.dateMembersSerializationMode; }
    //            set { this.dateMembersSerializationMode = value; }
    //        }

    //        //	BF 6.17.04	NAS2004 Vol3 - Notes Data Binding
    //        /// <summary>
    //        /// Resets all properties of the <see cref="AppointmentsDataBinding"/> object to their default values
    //        /// </summary>
    //        [EditorBrowsable(EditorBrowsableState.Advanced)]
    //        public override void Reset()
    //        {
    //            this.AllPropertiesMember = string.Empty;
    //            this.AllPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
    //            this.DataKeyMember = string.Empty;
    //            this.DateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;
    //            this.DateTimeFormatInfo = null;
    //            this.DateTimeFormatString = null;
    //            this.DescriptionMember = string.Empty;
    //            this.EndDateTimeMember = string.Empty;
    //            this.OriginalStartDateTimeMember = string.Empty;
    //            this.OwnerKeyMember = string.Empty;
    //            this.RecurrenceIdMember = string.Empty;
    //            this.RecurrenceMember = string.Empty;
    //            this.ReminderEnabledMember = string.Empty;
    //            this.ReminderIntervalMember = string.Empty;
    //            this.ReminderUnitsMember = string.Empty;
    //            this.StartDateTimeMember = string.Empty;
    //            this.SubjectMember = string.Empty;
    			
    //            base.Reset();
    //        }

    //        //	BF 10/26/06	BR17064
    //        #region OnDataSourceRemoved
    //        /// <summary>
    //        /// Called when the data source has been removed.
    //        /// </summary>
    //        internal protected override void OnDataSourceRemoved()
    //        {
    //            UltraCalendarInfo calendarInfo = this.OwnerCalendarInfo;
    //            if ( calendarInfo != null )
    //                calendarInfo.Appointments.ClearHelper( false );
    //        }
    //        #endregion OnDataSourceRemoved

    //    }
    //    #endregion AppointmentsDataBinding class
    #endregion Obsolete code

    #region ActivitiesDataBindingBase class
    /// <summary>
    /// Base class for the <see cref="NotesDataBinding"/> and <see cref="AppointmentsDataBinding"/> classes.
    /// </summary>
    public class ActivitiesDataBindingBase : DataBindingsBase
    {
        #region Constants
        
        private const DateSerializationMode defaultDateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;
        private const DateSerializationMode defaultAllPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
        #endregion Constants

        #region Member variables
        private string dateTimeFormatString = null;
		private IFormatProvider dateTimeFormatInfo = null;
		private DateSerializationMode allPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
		private DateSerializationMode dateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;
		private DateDeserializationMode dateMembersDeserializationMode = DateDeserializationMode.Default;
        #endregion Member variables

        #region Constructor
        /// <summary>
		/// Constructor.
		/// </summary>
        /// <param name="ownerCalendarInfo">The owning UltraCalendarInfo</param>
		/// <param name="size">The size</param>
		internal ActivitiesDataBindingBase(UltraCalendarInfo ownerCalendarInfo, int size) : base( ownerCalendarInfo, size ){}
        #endregion Constructor

        #region Abstract methods (not implemented)
        //
        //  Derived classes should be implementing these
        //
        /// <summary>
        /// Not implemented.
        /// </summary>
        protected override PropertyDescriptor ItemPropertyDescriptorCreator(Type type, string name)
        {
            throw new NotImplementedException();
        }

        internal override DataBoundItemsBase ItemList
        {
            get { throw new NotImplementedException(); }
        }

        internal override IDataBoundItem DataBoundItemCreator(int listIndex, object listObject, out bool isValidItem)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Not implemented.
        /// </summary>
        protected internal override void OnDataSourceRemoved()
        {
            throw new NotImplementedException();
        }
        #endregion Abstract methods (not implemented)

		#region Properties

			#region DateMembersSerializationMode
		/// <summary>
		/// Gets/sets the serialization mode for the DateTime members.
		/// </summary>
		public virtual DateSerializationMode DateMembersSerializationMode
		{
			get { return this.dateMembersSerializationMode; }
			set
            {
                if ( value != this.dateMembersSerializationMode )
                {
                    this.dateMembersSerializationMode = value;
                    this.NotifyPropChange( PropertyIds.DateMembersSerializationMode );
                }
            }
		}

		/// <summary>
		/// Indicates whether the <see cref="DateMembersSerializationMode"/> property needs to be serialized.
		/// </summary>
		/// <returns>True when the contents of the <see cref="DateMembersSerializationMode"/> property needs to be serialized.</returns>
		/// <seealso cref="DateMembersSerializationMode"/>
		protected bool ShouldSerializeDateMembersSerializationMode() { return this.DateMembersSerializationMode != defaultDateMembersSerializationMode; }
 
		/// <summary>
		/// Resets the <see cref="DateMembersSerializationMode"/> property to its default value.
		/// </summary>
		/// <seealso cref="DateMembersSerializationMode"/>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public void ResetDateMembersSerializationMode() { this.DateMembersSerializationMode = defaultDateMembersSerializationMode; }
			#endregion DateMembersSerializationMode

			#region DateMembersDeserializationMode
		/// <summary>
		/// Gets/sets the deserialization mode for the DateTime members.
		/// </summary>
        /// <seealso cref="DateDeserializationMode"/>
        /// <see cref="DateMembersSerializationMode"/>
		[LocalizedDescription("ActivitiesDataBindingBase_P_DateMembersDeserializationMode")]
		[LocalizedCategory("LC_Behavior")]
		public DateDeserializationMode DateMembersDeserializationMode
		{
			get { return this.dateMembersDeserializationMode; }
			set
            {
                if ( value != this.dateMembersDeserializationMode )
                {
                    this.dateMembersDeserializationMode = value;
                    this.NotifyPropChange( PropertyIds.DateMembersDeserializationMode );
                }
            }
		}

		/// <summary>
		/// Indicates whether the <see cref="DateMembersDeserializationMode"/> property needs to be serialized.
		/// </summary>
		/// <returns>True when the contents of the <see cref="DateMembersDeserializationMode"/> property needs to be serialized.</returns>
		/// <seealso cref="DateMembersDeserializationMode"/>
		protected bool ShouldSerializeDateMembersDeserializationMode() { return this.DateMembersDeserializationMode != DateDeserializationMode.Default; }
 
		/// <summary>
		/// Resets the <see cref="DateMembersDeserializationMode"/> property to its default value.
		/// </summary>
		/// <seealso cref="DateMembersDeserializationMode"/>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public void ResetDateMembersDeserializationMode() { this.DateMembersDeserializationMode = DateDeserializationMode.Default; }
			#endregion DateMembersDeserializationMode

			#region AllPropertiesMemberSerializationMode
		/// <summary>
		/// Gets/sets the serialization mode for the AllPropertiesMember.
		/// </summary>
		public virtual DateSerializationMode AllPropertiesMemberSerializationMode
		{
			get { return this.allPropertiesMemberSerializationMode; }
			set
            {
                if ( value != this.allPropertiesMemberSerializationMode )
                {
                    this.allPropertiesMemberSerializationMode = value;
                    this.NotifyPropChange( PropertyIds.AllPropertiesMemberSerializationMode );
                }
            }
        }

		/// <summary>
		/// Indicates whether the <see cref="AllPropertiesMemberSerializationMode"/> property needs to be serialized.
		/// </summary>
		/// <returns>True when the contents of the <see cref="AllPropertiesMemberSerializationMode"/> property needs to be serialized.</returns>
		/// <seealso cref="AllPropertiesMemberSerializationMode"/>
		protected bool ShouldSerializeAllPropertiesMemberSerializationMode() { return this.AllPropertiesMemberSerializationMode != defaultAllPropertiesMemberSerializationMode; }
 
		/// <summary>
		/// Resets the <see cref="AllPropertiesMemberSerializationMode"/> property to its default value.
		/// </summary>
		/// <seealso cref="AllPropertiesMemberSerializationMode"/>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public void ResetAllPropertiesMemberSerializationMode() { this.AllPropertiesMemberSerializationMode = defaultAllPropertiesMemberSerializationMode; }
			#endregion AllPropertiesMemberSerializationMode

			#region DateTimeFormatString
		/// <summary>
		/// Gets/sets the serialization mode for the DateTime members.
		/// </summary>
		public virtual string DateTimeFormatString
		{
			get { return this.dateTimeFormatString; }
			set
            {
                if ( value != this.dateTimeFormatString )
                {
                    this.dateTimeFormatString = value;
                    this.NotifyPropChange( PropertyIds.DateTimeFormatString );
                }
            }
		}

		/// <summary>
		/// Indicates whether the <see cref="DateTimeFormatString"/> property needs to be serialized.
		/// </summary>
		/// <returns>True when the contents of the <see cref="DateTimeFormatString"/> property needs to be serialized.</returns>
		/// <seealso cref="DateTimeFormatString"/>
		protected bool ShouldSerializeDateTimeFormatString() { return this.DateTimeFormatString != null; }
 
		/// <summary>
		/// Resets the <see cref="DateTimeFormatString"/> property to its default value.
		/// </summary>
		/// <seealso cref="DateTimeFormatString"/>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public void ResetDateTimeFormatString() { this.DateTimeFormatString = null; }
			#endregion DateTimeFormatString

			#region DateTimeFormatInfo
		/// <summary>
		/// Gets/sets the serialization mode for the DateTime members.
		/// </summary>
		public virtual IFormatProvider DateTimeFormatInfo
		{
			get { return this.dateTimeFormatInfo; }
			set
            {
                if ( value != this.dateTimeFormatInfo )
                {
                    this.dateTimeFormatInfo = value;
                    this.NotifyPropChange( PropertyIds.DateTimeFormatInfo );
                }
            }
		}

		/// <summary>
		/// Indicates whether the <see cref="DateTimeFormatInfo"/> property needs to be serialized.
		/// </summary>
		/// <returns>True when the contents of the <see cref="DateTimeFormatInfo"/> property needs to be serialized.</returns>
		/// <seealso cref="DateTimeFormatInfo"/>
		protected bool ShouldSerializeDateTimeFormatInfo() { return this.DateTimeFormatInfo != null; }
 
		/// <summary>
		/// Resets the <see cref="DateTimeFormatInfo"/> property to its default value.
		/// </summary>
		/// <seealso cref="DateTimeFormatInfo"/>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public void ResetDateTimeFormatInfo() { this.DateTimeFormatInfo = null; }
			#endregion DateTimeFormatInfo

            #region IsMemberPropertySet
        private bool IsMemberPropertySet
        {
            get
            {
                string[] propertyNames = this.PropertyNames;
                int propertyNameCount = propertyNames == null ? 0 : propertyNames.Length;
                bool propertyNameSet = false;
                for ( int i = 0; i < propertyNameCount; i ++ )
                {
                    if ( string.IsNullOrEmpty(propertyNames[i]) == false )
                    {
                        propertyNameSet = true;
                        break;
                    }
                }

                return propertyNameSet;
            }
        }
            #endregion IsMemberPropertySet

		#endregion Properties

        #region Methods

            #region ConvertDateTimeHelper
        internal object ConvertDateTimeHelper( object source, DateSerializationMode serializationMode, DateDeserializationMode deserializationMode )
        {
            bool validConversion;

            Type destinationType = typeof(DateTime);

			object convertedValue = DataBindingUtils.DataBindConvert(source, destinationType, CultureInfo.InvariantCulture, 
				this.DateTimeFormatString, this.DateTimeFormatInfo, out validConversion);

            if ( convertedValue != null && convertedValue is DateTime )
            {
                DateTime dateValue = (DateTime)convertedValue;

                //  BR29935 - This is the behavior-changing line of code that fixes the bug:
                //  If the DateDeserializationMode property is set, remove the DateTimeKind
                //  so we don't convert a date that is already being expressed as a local time.
                if ( deserializationMode == DateDeserializationMode.IgnoreDateTimeKind )
                    dateValue = DateTime.SpecifyKind( dateValue, DateTimeKind.Unspecified );

                //  Convert the date to local time if it was expressed as a universal time.
                if ( serializationMode == DateSerializationMode.SerializeDatesAsUniversalTime )
                    dateValue = dateValue.ToLocalTime();

                convertedValue = dateValue;
            }

            return convertedValue;
		}
            #endregion ConvertDateTimeHelper

            #region ShouldSerialize
        /// <summary>
        /// Returns whether any properties have been set to non-default values.
        /// </summary>
        public override bool ShouldSerialize()
        {
            return  this.IsMemberPropertySet ||
                    this.ShouldSerializeAllPropertiesMemberSerializationMode() ||
                    this.ShouldSerializeDateMembersSerializationMode() ||
                    this.ShouldSerializeDateMembersDeserializationMode() ||
                    this.ShouldSerializeDateTimeFormatInfo() ||
                    this.ShouldSerializeDateTimeFormatString() ||
                    base.ShouldSerialize();
        }
            #endregion ShouldSerialize

            #region Reset
        /// <summary>
        /// Resets all proeprties of theis instance to their default values.
        /// </summary>
        public override void Reset()
        {
            this.ResetAllPropertiesMemberSerializationMode();
            this.ResetDateMembersSerializationMode();
            this.ResetDateMembersDeserializationMode();
            this.ResetDateTimeFormatString();
            this.ResetDateTimeFormatInfo();
            base.Reset();
        }
            #endregion Reset

        #endregion Methods
    }

    #endregion ActivitiesDataBindingBase class

    #region NotesDataBinding class
    /// <summary>
	/// Class that handles data binding for the <see cref="UltraCalendarInfo"/> component's <see cref="UltraCalendarInfo.Notes"/> collection.
	/// </summary>
	/// <remarks>
	/// <p class="body">
	/// The <b>NAS2004 Volume 3</b> release of UltraWinSchedule includes support for data binding the UltraCalendarInfo component's Notes collection.
	/// The Notes collection can be populated from the contents of any IList implementor, for example, a DataTable. To identify the names of the members
	/// that contain the property values for each Note object, the <see cref="NotesDataBinding.DateMember"/> and the <see cref="NotesDataBinding.DescriptionMember"/>
	/// properties must be set. The contents of the fields that correspond to these members supply the values for the Note object's <see cref="Note.Date"/> and
	/// <see cref="Note.Description"/> properties, respectively. Each item in the list (for example, the individual rows in a DataTable) represents a member of
	/// the Notes collection. As property values of the Note object change, the new value is written to the corresponding member of the data source.
	/// </p>
	/// </remarks>
	/// <seealso cref="UltraCalendarInfo.DataBindingsForNotes"/>
	/// <seealso cref="UltraCalendarInfo.Notes"/>
	/// <seealso cref="Note"/>
	/// <seealso cref="NotesCollection"/>
	/// <seealso cref="AppointmentsDataBinding"/>
	/// <seealso cref="OwnersDataBinding"/>
	[TypeConverter( typeof( ExpandableObjectConverter ) )]
	public class NotesDataBinding: ActivitiesDataBindingBase
	{
		#region Member variables

		private NotesAdapter notesAdapter = null;

		private static PropertyIds[] notificationMapping = 
			new PropertyIds[] 
			{ 
				PropertyIds.AllPropertiesMember, 
				PropertyIds.DateMember, 
				PropertyIds.DescriptionMember, 
				PropertyIds.OwnerKeyMember, 
				PropertyIds.DataKeyMember 
			};

		#endregion Member variables

		#region Constructor
		/// <summary>
		/// (Internal) Creates a new instance of the <see cref="NotesDataBinding"/> class.
		/// </summary>
		/// <param name="ownerCalendarInfo">The <see cref="UltraCalendarInfo"/> component whose <see cref="UltraCalendarInfo.Notes"/> collection is to be bound.</param>
		internal NotesDataBinding(UltraCalendarInfo ownerCalendarInfo): 
				 base(ownerCalendarInfo, Enum.GetNames( typeof(NotesDataBindingPropertyId) ).Length - 4)
		{
			this.notesAdapter = new NotesAdapter( this.OwnerCalendarInfo.Notes );
		}
		#endregion Constructor

		#region NotesAdapter class (Private)
		private class NotesAdapter: DataBoundItemsBase
		{
			#region Member variables

			private NotesCollection notes;
			
			#endregion Member variables

			#region Constructor
			/// <summary>
			/// Creates a new instance of the NotesAdapter class.
			/// </summary>
			/// <param name="notes">A reference to the Notes collection for which this instance provides data.</param>
			public NotesAdapter( NotesCollection notes )
			{
				this.notes = notes;
			}
			#endregion Constructor

			#region DirtyList
			/// <summary>
			/// Overridden; called in response to the ListChanged event.
			/// </summary>
			/// <param name="dirtyEveryItem">Boolean specifying whether every member of the Notes collection is to be dirtied.</param>
			public override void DirtyList( bool dirtyEveryItem )
			{
				if( dirtyEveryItem )
					this.notes.DirtyAllNotes();

				//	Dirty version number, so that any subset collections are rebuilt
				this.notes.CollectionChanged();

				//	Issue a property change notification
				this.notes.NotifyPropChange(PropertyIds.Note);
			}
			#endregion DirtyList

			#region Clear
			/// <summary>
			/// Overridden; called when the Notes collection is to be cleared.
			/// </summary>
			public override void Clear()
			{
				this.notes.ClearHelper(false);
			}
			#endregion Clear

			#region Add
			/// <summary>
			/// Overridden; called when a new member is to be added to the Notes collection.
			/// </summary>
			public override int Add( object value )
			{
				//return this.notes.AddNoteHelper( value as Note, false );
				return this.notes.AddNoteHelper( value as Note );
			}
			#endregion Add

			#region Count
			/// <summary>
			/// Overridden; returns the count of the Notes collection
			/// </summary>
			public override int Count
			{ 
				get { return this.notes.Count; } 
			}
			#endregion Count

			#region RemoveAt
			/// <summary>
			/// Overridden; removes the member of the Notes collection whose
			/// ordinal position corresponds to the specified index.
			/// </summary>
			public override void RemoveAt( int index )
			{
				//	BF 5/31/06	BR12852
				//this.notes.RemoveHelper( this.notes[index] );
				this.notes.RemoveHelper( this.notes[index], false );
			}
			#endregion RemoveAt

			#region Indexer ('this' operator)
			/// <summary>
			/// Overridden; returns the member of the Notes collection whose
			/// ordinal position corresponds to the specified index.
			/// </summary>
			public override object this[int index]
			{
				get { return this.notes[index]; }
			}
			#endregion Indexer ('this' operator)

			#region GetEnumerator
            /// <summary>
            /// IEnumerable Interface Implementation returns a type safe enumerator
            /// </summary>		
            /// <returns>A type safe enumerator.</returns>		
            public override IEnumerator GetEnumerator()
			{
				return this.notes.GetEnumerator();
			}
			#endregion GetEnumerator
		}
		#endregion NotesAdapter class (Private)

		#region ItemList
		/// <summary>
		/// Returns the NotesAdapter instance associated with this NotesDataBinding instance.
		/// </summary>
		internal override DataBoundItemsBase ItemList 
		{ 
			get
			{
				return this.notesAdapter;
			}
		}
		#endregion ItemList

		#region ItemPropertyDescriptorCreator
		/// <summary>
		/// Creator for class-specific property descriptor.
		/// </summary>
        /// <param name="type">The type</param>
        /// <param name="name">The name</param>
		protected override PropertyDescriptor ItemPropertyDescriptorCreator( Type type, string name )
		{
			throw new Exception( SR.GetString("LE_AppointmentsDataBinding_CannotBind") );
		}
		#endregion ItemPropertyDescriptorCreator

		#region DataBoundItemCreator
		/// <summary>
		/// Creats a new instance of the Note object
		/// </summary>
		/// <param name="listIndex">The value for the BindingListIndex property of the Note to be created.</param>
		/// <param name="listObject">The value for the BindingListObject property of the Note to be created.</param>
		/// <param name="isValidItem">On return, specifies whether the Note was successfully created.</param>
		/// <returns></returns>
		internal override IDataBoundItem DataBoundItemCreator( int listIndex, object listObject, out bool isValidItem )
		{
			Note note = new Note( this.OwnerCalendarInfo, listIndex, listObject );

			isValidItem = ((IDataBoundItem)note).DirtyItem( true );

			note.SubObjectPropChanged += new SubObjectPropChangeEventHandler( note.OnThisObjectChanged );

			return ((IDataBoundItem)note);
		}
		#endregion DataBoundItemCreator

		#region GetPropertyName
		/// <summary>
		/// Returns the property name that corresponds to the specified NotesDataBindingPropertyId.
		/// </summary>
		/// <param name="currentPropertyId">The NotesDataBindingPropertyId whose property name is to be returned.</param>
		private string GetPropertyName( NotesDataBindingPropertyId currentPropertyId )
		{
			return this.PropertyNames[(int)currentPropertyId];
		}
		#endregion GetPropertyName

		#region SetPropertyName
		/// <summary>
		/// Sets the property name that corresponds to the specified NotesDataBindingPropertyId.
		/// </summary>
		/// <param name="name">The property name to be returned.</param>
		/// <param name="currentPropertyId">The NotesDataBindingPropertyId whose property name is to be returned.</param>
		private void SetPropertyName( string name, NotesDataBindingPropertyId currentPropertyId )
		{
			string newValue = DataBindingUtils.StripMember(name);

			if( newValue != this.PropertyNames[(int)currentPropertyId] )
			{
				//	BF 11/1/05	BR07410
				string oldValue = this.PropertyNames[(int)currentPropertyId];
				bool dirtyAll = (oldValue != null && oldValue.Length > 0);

				this.PropertyNames[(int)currentPropertyId] = newValue;

				this.InitializePropertyDescriptors();

				//	BF 8/3/05	BR05397
				//	BF 11/1/05	BR07410
				//if ( this.OwnerCalendarInfo != null )
				if ( dirtyAll && this.OwnerCalendarInfo != null )
				{
					NotesCollection notes = this.OwnerCalendarInfo.Notes;
					for ( int i = 0; i < notes.Count; i ++ )
					{
						Note note = notes[i];
						note.DirtyBoundPropertyValue( currentPropertyId );
					}
				}

				this.NotifyPropChange( NotesDataBinding.notificationMapping[ (int)currentPropertyId ] );
			}		
		}
		#endregion SetPropertyName

		#region Properties
			
			#region DateMember
		/// <summary>
		/// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.Date"/> property when data bound.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_DateMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string DateMember
		{
			get
			{
				return this.GetPropertyName( NotesDataBindingPropertyId.DateMember );
			}
			set
			{
				this.SetPropertyName( value, NotesDataBindingPropertyId.DateMember );
			}
		}	
			#endregion DateMember

			#region DescriptionMember
		/// <summary>
		/// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.Description"/> property when data bound.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_DescriptionMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string DescriptionMember
		{
			get
			{
				return this.GetPropertyName( NotesDataBindingPropertyId.DescriptionMember );
			}
			set
			{
				this.SetPropertyName( value, NotesDataBindingPropertyId.DescriptionMember );
			}
		}	
			#endregion DescriptionMember

			#region OwnerKeyMember
		/// <summary>
		/// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.OwnerKey"/> property when data bound.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_OwnerKeyMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string OwnerKeyMember
		{
			get
			{
				return this.GetPropertyName( NotesDataBindingPropertyId.OwnerKeyMember );
			}
			set
			{
				this.SetPropertyName( value, NotesDataBindingPropertyId.OwnerKeyMember );
			}
		}	
			#endregion OwnerKeyMember

			#region DataKeyMember
		/// <summary>
		/// Gets/sets the field to use to initialize the <see cref="Note"/>'s <see cref="Note.DataKey"/> property when data bound.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_DataKeyMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string DataKeyMember
		{
			get
			{
				return this.GetPropertyName( NotesDataBindingPropertyId.DataKeyMember );
			}
			set
			{
				this.SetPropertyName( value, NotesDataBindingPropertyId.DataKeyMember );
			}
		}	
			#endregion DataKeyMember

			#region AllPropertiesMember
		/// <summary>
		/// Gets/sets the field to use to initialize the <see cref="Note"/> from serialized data.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_AllPropertiesMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string AllPropertiesMember
		{
			get
			{
				return this.GetPropertyName( NotesDataBindingPropertyId.AllPropertiesMember );
			}
			set
			{
				this.SetPropertyName( value, NotesDataBindingPropertyId.AllPropertiesMember );
			}
		}	
			#endregion AllPropertiesMember

			#region DateTimeFormatString
		/// <summary>
		/// Gets/sets the format string used for DateTime serialization.
		/// </summary>
		/// <seealso cref="NotesDataBinding.DateTimeFormatInfo"/>
		[LocalizedDescription("NotesDataBinding_P_DateTimeFormatString")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		public override string DateTimeFormatString
		{
			get { return base.DateTimeFormatString; }
            set { base.DateTimeFormatString = value; }
		}	
			#endregion DateTimeFormatString

			#region DateTimeFormatInfo
		/// <summary>
		/// Gets/sets the IFormatProvider implementor used for DateTime serialization.
		/// </summary>
		/// <seealso cref="NotesDataBinding.DateTimeFormatString"/>
		[Editor(typeof(FormatProviderEditor), typeof(UITypeEditor))]
		[DesignerSerializationVisibility( DesignerSerializationVisibility.Visible )]
		[TypeConverter(typeof( FormatProviderTypeConverter ))]
		[LocalizedDescription("LD_DataBinding_P_FormatProvider")]
		[LocalizedCategory("LC_Behavior")]
		public override IFormatProvider DateTimeFormatInfo
		{
			get { return base.DateTimeFormatInfo; }
            set { base.DateTimeFormatInfo = value; }
		}
			#endregion DateTimeFormatInfo

			#region AllPropertiesMemberSerializationMode
		/// <summary>
		/// Gets/sets the date serialization mode for the <see cref="NotesDataBinding.AllPropertiesMember"/>.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_AllPropertiesMemberSerializationMode")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(DateSerializationMode.SerializeDatesAsUniversalTime)]
		public override DateSerializationMode AllPropertiesMemberSerializationMode
		{
			get { return base.AllPropertiesMemberSerializationMode; }
			set { base.AllPropertiesMemberSerializationMode = value; }
		}
			#endregion AllPropertiesMemberSerializationMode

			#region DateMembersSerializationMode
		/// <summary>
		/// Date serialization mode for the <see cref="Note.Date"/> property.
		/// </summary>
		[LocalizedDescription("NotesDataBinding_P_DateMembersSerializationMode")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(DateSerializationMode.SerializeDatesAsLocalTime)]
		public override DateSerializationMode DateMembersSerializationMode
		{
			get { return base.DateMembersSerializationMode; }
			set { base.DateMembersSerializationMode = value; }
		}
			#endregion DateMembersSerializationMode

		#endregion Properties

		#region Methods

			#region ShouldSerialize
		/// <summary>
		/// Returns whether the <see cref="NotesDataBinding"/> object needs to be serialized.
		/// </summary>
        /// <returns>true if this object needs to be serialized; otherwise, false</returns>
        public override bool ShouldSerialize()
		{
			return base.ShouldSerialize();										
		}
			#endregion ShouldSerialize

			#region Reset
		/// <summary>
		/// Resets all properties of the <see cref="NotesDataBinding"/> object to their default values
		/// </summary>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public override void Reset()
		{
			this.AllPropertiesMember = string.Empty;
			this.DataKeyMember = string.Empty;
			this.DescriptionMember = string.Empty;
			this.OwnerKeyMember = string.Empty;
			
			base.Reset();
		}

			#endregion Reset

		//	BF 10/26/06	BR17064
			#region OnDataSourceRemoved
		/// <summary>
		/// Called when the data source has been removed.
		/// </summary>
		internal protected override void OnDataSourceRemoved()
		{
			UltraCalendarInfo calendarInfo = this.OwnerCalendarInfo;
			if ( calendarInfo != null )
				calendarInfo.Notes.ClearHelper( false );
		}
			#endregion OnDataSourceRemoved

		#endregion	Methods
	}
	#endregion NotesDataBinding class

    #region AppointmentsDataBinding
    /// <summary>
	/// Scheduler data bindings for appointments.
	/// </summary>
	[TypeConverter( typeof( ExpandableObjectConverter ) )]
	public class AppointmentsDataBinding: ActivitiesDataBindingBase
    {
        #region Member variables
        private AppointmentsAdapter appointmentsAdapter = null;

		private static PropertyIds[] notificationMapping = 
			new PropertyIds[] 
			{ 
				PropertyIds.StartDateTimeMember, 
				PropertyIds.EndDateTimeMember, 
				PropertyIds.SubjectMember, 
				PropertyIds.DescriptionMember, 
				PropertyIds.AllDayEventMember, 
				PropertyIds.ReminderEnabledMember, 
				PropertyIds.ReminderIntervalMember, 
				PropertyIds.ReminderUnitsMember, 
				PropertyIds.OwnerKeyMember,
				PropertyIds.DataKeyMember, 
				PropertyIds.RecurrenceMember, 
				PropertyIds.RecurrenceIdMember, 
				PropertyIds.OriginalStartDateTimeMember, 
				PropertyIds.AllPropertiesMember 
			};
        #endregion Member variables

        #region Constructor
        internal AppointmentsDataBinding(UltraCalendarInfo ownerCalendarInfo): 
			base(ownerCalendarInfo, Enum.GetNames( typeof(AppointmentsDataBindingPropertyId) ).Length - 4)
		{
			this.appointmentsAdapter = new AppointmentsAdapter( this.OwnerCalendarInfo.Appointments );
		}
        #endregion Constructor

        #region AppointmentsAdapter class
        private class AppointmentsAdapter: DataBoundItemsBase
		{
			private AppointmentsCollection appointments;

			public AppointmentsAdapter(AppointmentsCollection appointments)
			{
				this.appointments = appointments;
			}

			public override void DirtyList(bool dirtyEveryItem)
			{
				if(dirtyEveryItem)
					this.appointments.DirtyAllAppointments(false);

				
				this.appointments.BumpVersionNumber();

				this.appointments.NotifyPropChange(PropertyIds.Appointment);
			}

			public override void Clear()
			{
				this.appointments.ClearHelper(false);
			}

			public override int Add(object value)
			{
				return this.appointments.AddAppointmentHelper(value as Appointment, false);
			}

			public override int Count
			{ 
				get { return this.appointments.Count; } 
			}

			public override void RemoveAt(int index)
			{
				this.appointments.RemoveHelper( this.appointments[index], false );
			}

			public override object this[int index]
			{
				get { return this.appointments[index]; }
			}

            /// <summary>
            /// IEnumerable Interface Implementation returns a type safe enumerator
            /// </summary>		
            /// <returns>A type safe enumerator.</returns>		
            public override IEnumerator GetEnumerator()
			{
				return this.appointments.GetEnumerator();
			}
		}
        #endregion AppointmentsAdapter class

        #region Abstract method overrides

        /// <summary>
		/// Underlying list of items.
		/// </summary>
		internal override DataBoundItemsBase ItemList 
		{ 
			get
			{
				return this.appointmentsAdapter;
			}
		}

		/// <summary>
		/// Creator for class-specific property descriptor.
		/// </summary>
        /// <param name="type">The type</param>
        /// <param name="name">The name</param>
		protected override PropertyDescriptor ItemPropertyDescriptorCreator(Type type, string name)
		{
			throw new Exception( SR.GetString("LE_AppointmentsDataBinding_CannotBind") );
		}

		internal override IDataBoundItem DataBoundItemCreator(int listIndex, object listObject, out bool isValidItem)
		{
			Appointment appt = new Appointment(this.OwnerCalendarInfo, listIndex, listObject);

			isValidItem = ((IDataBoundItem)appt).DirtyItem(true);

			//	BF 3/18/05	BR02914
			//	This is no longer needed
			//appt.SubObjectPropChanged += new SubObjectPropChangeEventHandler( appt.OnThisObjectChanged );

			return ((IDataBoundItem)appt);
		}
        #endregion Abstract method overrides

        #region Methods

        private string GetPropertyName(AppointmentsDataBindingPropertyId currentPropertyId)
		{
			return this.PropertyNames[(int)currentPropertyId];
		}

		private void SetPropertyName(string name, AppointmentsDataBindingPropertyId currentPropertyId)
		{
			string newValue = DataBindingUtils.StripMember(name);

			if(newValue != this.PropertyNames[(int)currentPropertyId])
			{
				//	BF 11/1/05	BR07410
				string oldValue = this.PropertyNames[(int)currentPropertyId];
				bool dirtyAll = (oldValue != null && oldValue.Length > 0);

				this.PropertyNames[(int)currentPropertyId] = newValue;

				this.InitializePropertyDescriptors();

				//	BF 8/3/05	BR05397
				//
				//	In the BR05397 scenario, the SubjectMember field was changed during
				//	the lifetime of the application, so that a different value could be
				//	displayed. We already had the plumbing in place to refresh the property
				//	descriptor, so all that needed to be done was to mark the property dirty
				//	for each appointment in the collection.
				//
				//	BF 11/1/05	BR07410
				
				//	Since we didn't used to do this prior to the fix for BR05397, this
				//	shouldn't cause any other problems, and it doesn't break BR05397
				//	because if the ***Member property wasn't previously set the Appointment
				//	property value cannot possibly be stale.
				//if ( this.OwnerCalendarInfo != null )
				if ( dirtyAll && this.OwnerCalendarInfo != null )
				{
					AppointmentsCollection appointments = this.OwnerCalendarInfo.Appointments;
					for ( int i = 0; i < appointments.Count; i ++ )
					{
						Appointment appointment = appointments[i];
						appointment.DirtyBoundPropertyValue( currentPropertyId );
					}
				}

				this.NotifyPropChange( AppointmentsDataBinding.notificationMapping[ (int) currentPropertyId ] );
			}		
		}
        #endregion Methods

        #region Properties

            #region StartDateTimeMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s start date when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_StartDateTimeMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string StartDateTimeMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.StartDateTimeMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.StartDateTimeMember );
			}
		}	
            #endregion StartDateTimeMember

            #region EndDateTimeMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s end date when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_EndDateTimeMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string EndDateTimeMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.EndDateTimeMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.EndDateTimeMember );
			}
		}	
            #endregion EndDateTimeMember

            #region SubjectMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s subject property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_SubjectMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string SubjectMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.SubjectMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.SubjectMember );
			}
		}	
            #endregion SubjectMember

            #region DescriptionMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s description property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_DescriptionMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string DescriptionMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.DescriptionMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.DescriptionMember );
			}
		}	
            #endregion DescriptionMember

            #region AllDayEventMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s AllDayEvent property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_AllDayEventMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string AllDayEventMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.AllDayEventMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.AllDayEventMember );
			}
		}	
            #endregion AllDayEventMember

            #region ReminderEnabledMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s reminder when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_ReminderEnabledMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string ReminderEnabledMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.ReminderEnabledMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.ReminderEnabledMember );
			}
		}	
            #endregion ReminderEnabledMember

            #region ReminderIntervalMember
        /// <summary>
		/// Gets/sets the field to use to initialize the reminder�s DisplayInterval property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_ReminderIntervalMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string ReminderIntervalMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.ReminderIntervalMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.ReminderIntervalMember );
			}
		}	
            #endregion ReminderIntervalMember

            #region ReminderUnitsMember
        /// <summary>
		/// Gets/sets the field to use to initialize the reminder�s DisplayIntervalUnits property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_ReminderUnitsMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string ReminderUnitsMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.ReminderUnitsMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.ReminderUnitsMember );
			}
		}	
            #endregion ReminderUnitsMember

            #region OwnerKeyMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s owner property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_OwnerKeyMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string OwnerKeyMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.OwnerKeyMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.OwnerKeyMember );
			}
		}	
            #endregion OwnerKeyMember

            #region DataKeyMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s DataKey property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_DataKeyMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string DataKeyMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.DataKeyMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.DataKeyMember );
			}
		}	
            #endregion DataKeyMember

            #region RecurrenceMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s Recurrence property to new AppointmentRecurrence. Appointment variances don�t make use of this as they only contain reference to �root� recurrence.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_RecurrenceMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string RecurrenceMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.RecurrenceMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.RecurrenceMember );
			}
		}	
            #endregion RecurrenceMember

            #region RecurrenceIdMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s Recurrence property to existing AppointmentRecurrence in Root Appointment.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_RecurrenceIdMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string RecurrenceIdMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.RecurrenceIdMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.RecurrenceIdMember );
			}
		}	
            #endregion RecurrenceIdMember

            #region OriginalStartDateTimeMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment�s OriginalStartDateTime property when bound.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_OriginalStartDateTimeMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string OriginalStartDateTimeMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.OriginalStartDateTimeMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.OriginalStartDateTimeMember );
			}
		}	
            #endregion OriginalStartDateTimeMember

            #region AllPropertiesMember
        /// <summary>
		/// Gets/sets the field to use to initialize the appointment from serialized data.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_AllPropertiesMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string AllPropertiesMember
		{
			get
			{
				return this.GetPropertyName( AppointmentsDataBindingPropertyId.AllPropertiesMember );
			}
			set
			{
				this.SetPropertyName( value, AppointmentsDataBindingPropertyId.AllPropertiesMember );
			}
		}	
            #endregion AllPropertiesMember

            #region DateTimeFormatString
        /// <summary>
		/// Specifies format string used for DateTime serialization.
		/// </summary>
		/// <seealso cref="AppointmentsDataBinding.DateTimeFormatInfo"/>
		[LocalizedDescription("LD_AppointmentsDataBinding_P_DateTimeFormatString")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		public override string DateTimeFormatString
		{
			get { return base.DateTimeFormatString; }
			set { base.DateTimeFormatString = value; }
		}	
            #endregion DateTimeFormatString

    		#region DateTimeFormatInfo
		/// <summary>
		/// Gets/sets the IFormatProvider implementor used for DateTime serialization.
		/// </summary>
		/// <seealso cref="AppointmentsDataBinding.DateTimeFormatString"/>
		[Editor(typeof(FormatProviderEditor), typeof(UITypeEditor))]
		[DesignerSerializationVisibility( DesignerSerializationVisibility.Visible )]
		[TypeConverter(typeof( FormatProviderTypeConverter ))]
		[LocalizedDescription("LD_DataBinding_P_FormatProvider")]
		[LocalizedCategory("LC_Behavior")]
		public override IFormatProvider DateTimeFormatInfo
		{
			get { return base.DateTimeFormatInfo; }
			set { base.DateTimeFormatInfo = value; }
		}	
		    #endregion DateTimeFormatInfo

            #region AllPropertiesMemberSerializationMode
        /// <summary>
		/// Date serialization mode for <see cref="AppointmentsDataBinding.AllPropertiesMember"/>. Also used 
		/// for <see cref="AppointmentsDataBinding.RecurrenceMember"/>.
		/// </summary>
		[LocalizedDescription("LD_AppointmentsAdapter_P_AllPropertiesMemberSerializationMode")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(DateSerializationMode.SerializeDatesAsUniversalTime)]
		public override DateSerializationMode AllPropertiesMemberSerializationMode
		{
			get { return base.AllPropertiesMemberSerializationMode; }
			set { base.AllPropertiesMemberSerializationMode = value; }
		}
            #endregion AllPropertiesMemberSerializationMode

            #region DateMembersSerializationMode
        /// <summary>
		/// Date serialization mode for all date members (<see cref="AppointmentsDataBinding.StartDateTimeMember"/>,
		/// <see cref="AppointmentsDataBinding.EndDateTimeMember"/> and <see cref="AppointmentsDataBinding.OriginalStartDateTimeMember"/>).
		/// </summary>
		[LocalizedDescription("LD_AppointmentsAdapter_P_DateMembersSerializationMode")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(DateSerializationMode.SerializeDatesAsLocalTime)]
		public override DateSerializationMode DateMembersSerializationMode
		{
			get { return base.DateMembersSerializationMode; }
			set { base.DateMembersSerializationMode = value; }
		}
            #endregion DateMembersSerializationMode

        #endregion Properties

        #region Methods

            #region ShouldSerialize
        /// <summary>
		/// Indicates whether the <see cref="AppointmentsDataBinding"/> object needs to be persisted.
		/// </summary>
        /// <returns>true if this object needs to be serialized; otherwise, false</returns>
        public override bool ShouldSerialize()
		{
            return base.ShouldSerialize();
		}
            #endregion ShouldSerialize

            #region Reset
        /// <summary>
		/// Resets all properties of the <see cref="AppointmentsDataBinding"/> object to their default values
		/// </summary>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public override void Reset()
		{
			this.AllPropertiesMember = string.Empty;
			this.AllPropertiesMemberSerializationMode = DateSerializationMode.SerializeDatesAsUniversalTime;
			this.DataKeyMember = string.Empty;
			this.DateMembersSerializationMode = DateSerializationMode.SerializeDatesAsLocalTime;
			this.DateTimeFormatInfo = null;
			this.DateTimeFormatString = null;
			this.DescriptionMember = string.Empty;
			this.EndDateTimeMember = string.Empty;
			this.OriginalStartDateTimeMember = string.Empty;
			this.OwnerKeyMember = string.Empty;
			this.RecurrenceIdMember = string.Empty;
			this.RecurrenceMember = string.Empty;
			this.ReminderEnabledMember = string.Empty;
			this.ReminderIntervalMember = string.Empty;
			this.ReminderUnitsMember = string.Empty;
			this.StartDateTimeMember = string.Empty;
			this.SubjectMember = string.Empty;
			
			base.Reset();
		}
            #endregion Reset

		//	BF 10/26/06	BR17064
		    #region OnDataSourceRemoved
		/// <summary>
		/// Called when the data source has been removed.
		/// </summary>
		internal protected override void OnDataSourceRemoved()
		{
			UltraCalendarInfo calendarInfo = this.OwnerCalendarInfo;
			if ( calendarInfo != null )
				calendarInfo.Appointments.ClearHelper( false );
		}
		    #endregion OnDataSourceRemoved

        #endregion Methods
    }
    #endregion AppointmentsDataBinding

    #region OwnersDataBinding
    /// <summary>
	/// Scheduler data bindings for owners.
	/// </summary>
	[TypeConverter( typeof( ExpandableObjectConverter ) )]
	public class OwnersDataBinding: DataBindingsBase
	{
		private OwnersAdapter ownersAdapter = null;

		private static PropertyIds[] notificationMapping = 
			new PropertyIds[] 
			{ 
				PropertyIds.KeyMember, 
				PropertyIds.NameMember, 
				PropertyIds.EmailAddressMember, 
				PropertyIds.VisibleMember, 
				PropertyIds.AllPropertiesMember
			};

		internal OwnersDataBinding(UltraCalendarInfo ownerCalendarInfo): 
			base(ownerCalendarInfo, Enum.GetNames( typeof(OwnersDataBindingPropertyId) ).Length)
		{
			this.ownersAdapter = new OwnersAdapter( this.OwnerCalendarInfo.Owners );
		}

		private class OwnersAdapter: DataBoundItemsBase
		{
			private OwnersCollection owners;

			public OwnersAdapter(OwnersCollection owners)
			{
				this.owners = owners;
			}

			public override void DirtyList(bool dirtyEveryItem)
			{
				if(dirtyEveryItem)
					this.owners.DirtyAllOwners();

				
				this.owners.BumpVersion();

				this.owners.NotifyPropChange(PropertyIds.Owner);
			}

			public override void Clear()
			{
				
				
				
				
				
				this.owners.ClearHelper(false);
			}

			public override int Add(object value)
			{
				return this.owners.AddOwnerHelper(value as Owner);
			}

			public override int Count
			{ 
				get { return this.owners.Count; } 
			}

			public override void RemoveAt(int index)
			{
				this.owners.RemoveHelper( this.owners[index] );
			}

			public override object this[int index]
			{
				get { return this.owners[index]; }
			}

            /// <summary>
            /// IEnumerable Interface Implementation returns a type safe enumerator
            /// </summary>		
            /// <returns>A type safe enumerator.</returns>		
            public override IEnumerator GetEnumerator()
			{
				return this.owners.GetEnumerator();
			}
		}

		/// <summary>
		/// Underlying list of items.
		/// </summary>
		internal override DataBoundItemsBase ItemList 
		{ 
			get
			{
				return this.ownersAdapter;
			}
		}

		/// <summary>
		/// Creator for class-specific property descriptor.
		/// </summary>
        /// <param name="type">The type</param>
        /// <param name="name">The name</param>
		protected override PropertyDescriptor ItemPropertyDescriptorCreator(Type type, string name)
		{
			throw new Exception( SR.GetString("LE_OwnersDataBinding_CannotBind") );
		}

		/// <summary>
		/// Performs full synchronization between items and data source.
		/// </summary>
        /// <param name="itemList">The item list.</param>
        /// <param name="boundList">The bound list.</param>
        protected override void FullSynchronization(IList itemList, IList boundList)
		{
			base.FullSynchronization(itemList, boundList);

			// For owners, we additionally need to erase all appointments and notes 
			// which now don't have matching owners.
			this.OwnerCalendarInfo.Appointments.RemoveAllAppointmentsForOwnerInternal( null );
			this.OwnerCalendarInfo.Notes.RemoveAllNotesForOwnerInternal( null );

			
			this.OwnerCalendarInfo.Appointments.DirtyAllAppointments(true); 
		}

		internal override IDataBoundItem DataBoundItemCreator(int listIndex, object listObject, out bool isValidItem)
		{
			Owner owner = new Owner(this.OwnerCalendarInfo, listIndex, listObject);

			isValidItem = ((IDataBoundItem)owner).DirtyItem(true);

			owner.SubObjectPropChanged += new SubObjectPropChangeEventHandler( owner.OnThisObjectChanged );

			return ((IDataBoundItem)owner);
		}

		private string GetPropertyName(OwnersDataBindingPropertyId currentPropertyId)
		{
			return this.PropertyNames[(int)currentPropertyId];
		}

		private void SetPropertyName(string name, OwnersDataBindingPropertyId currentPropertyId)
		{
			string newValue = DataBindingUtils.StripMember(name);

			if(newValue != this.PropertyNames[(int)currentPropertyId])
			{
				//	BF 11/1/05	BR07410
				string oldValue = this.PropertyNames[(int)currentPropertyId];
				bool dirtyAll = (oldValue != null && oldValue.Length > 0);

				this.PropertyNames[(int)currentPropertyId] = newValue;

				this.InitializePropertyDescriptors();

				//	BF 8/3/05	BR05397
				//	BF 11/1/05	BR07410
				//if ( this.OwnerCalendarInfo != null )
				if ( dirtyAll && this.OwnerCalendarInfo != null )
				{
					OwnersCollection owners = this.OwnerCalendarInfo.Owners;
					for ( int i = 0; i < owners.Count; i ++ )
					{
						Owner owner = owners[i];
						owner.DirtyBoundPropertyValue( currentPropertyId );
					}
				}

				this.NotifyPropChange( OwnersDataBinding.notificationMapping[ (int) currentPropertyId ] );
			}		
		}

		/// <summary>
		/// Gets/sets the field to use to initialize the owner�s Key property when bound.
		/// </summary>
		[LocalizedDescription("LD_OwnersDataBinding_P_KeyMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string KeyMember
		{
			get
			{
				return this.GetPropertyName( OwnersDataBindingPropertyId.KeyMember );
			}
			set
			{
				this.SetPropertyName( value, OwnersDataBindingPropertyId.KeyMember );
			}
		}

		/// <summary>
		/// Gets/sets the field to use to initialize the owner�s Name property when bound.
		/// </summary>
		[LocalizedDescription("LD_OwnersDataBinding_P_NameMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string NameMember
		{
			get
			{
				return this.GetPropertyName( OwnersDataBindingPropertyId.NameMember );
			}
			set
			{
				this.SetPropertyName( value, OwnersDataBindingPropertyId.NameMember );
			}
		}

		/// <summary>
		/// Gets/sets the field to use to initialize the owner�s EmailAddress property when bound.
		/// </summary>
		[LocalizedDescription("LD_OwnersDataBinding_P_EmailAddressMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string EmailAddressMember
		{
			get
			{
				return this.GetPropertyName( OwnersDataBindingPropertyId.EmailAddressMember );
			}
			set
			{
				this.SetPropertyName( value, OwnersDataBindingPropertyId.EmailAddressMember );
			}
		}

		/// <summary>
		/// Gets/sets the field to use to initialize the owner�s Visible property when bound.
		/// </summary>
		[LocalizedDescription("LD_OwnersDataBinding_P_VisibleMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string VisibleMember
		{
			get
			{
				return this.GetPropertyName( OwnersDataBindingPropertyId.VisibleMember );
			}
			set
			{
				this.SetPropertyName( value, OwnersDataBindingPropertyId.VisibleMember );
			}
		}	

		/// <summary>
		/// Gets/sets the field to use to initialize the owner from serialized data.
		/// </summary>
		[LocalizedDescription("LD_OwnersDataBinding_P_AllPropertiesMember")]
		[LocalizedCategory("LC_Data")]
		[DefaultValue(null)]
		[Editor( "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing" )]
		public string AllPropertiesMember
		{
			get
			{
				return this.GetPropertyName( OwnersDataBindingPropertyId.AllPropertiesMember );
			}
			set
			{
				this.SetPropertyName( value, OwnersDataBindingPropertyId.AllPropertiesMember );
			}
		}	

		/// <summary>
		/// Indicates whether the <see cref="OwnersDataBinding"/> object needs to be persisted.
		/// </summary>
        /// <returns>true if this object needs to be serialized; otherwise, false</returns>
        public new bool ShouldSerialize()
		{
			if(this.KeyMember!=null || this.NameMember!=null || this.EmailAddressMember!=null ||
				this.VisibleMember!=null || this.AllPropertiesMember!=null || base.ShouldSerialize())
				return true;
			else
				return false;
		}

		//	BF 6.17.04	NAS2004 Vol3 - Notes Data Binding
		/// <summary>
		/// Resets all properties of the <see cref="OwnersDataBinding"/> object to their default values
		/// </summary>
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		public override void Reset()
		{
			this.AllPropertiesMember = string.Empty;
			this.EmailAddressMember = string.Empty;
			this.KeyMember = string.Empty;
			this.NameMember = string.Empty;
			this.VisibleMember = string.Empty;

			base.Reset();
		}

		//	BF 10/26/06	BR17064
		#region OnDataSourceRemoved
		/// <summary>
		/// Called when the data source has been removed.
		/// </summary>
		internal protected override void OnDataSourceRemoved()
		{
			UltraCalendarInfo calendarInfo = this.OwnerCalendarInfo;
			if ( calendarInfo != null )
				calendarInfo.Owners.ClearHelper( false );
		}
		#endregion OnDataSourceRemoved

        //  BF 2/20/08  BR29761
        #region ClearKeys
        internal override void ClearKeys()
        {
            if ( this.ownersAdapter != null )
            {
                for ( int i = 0; i < this.ownersAdapter.Count; i ++ )
                {
                    Owner owner = this.ownersAdapter[i] as Owner;
                    if ( owner == null || owner.IsUnassigned )
                        continue;

                    Guid guid = Guid.NewGuid();
                    owner.SetKeyHelper( guid.ToString(), false );
                }
            }
        }
        #endregion ClearKeys
	}
    #endregion OwnersDataBinding

	//	BF 5.17.04	UWS968
    //	Made this public so we can identify the property in the AppointmentDataError event
    #region AppointmentsDataBindingPropertyId enumeration
    /// <summary>
	/// Values that uniquely identify each bindable property of the <see cref="Infragistics.Win.UltraWinSchedule.Appointment"/> object.
	/// </summary>
	public enum AppointmentsDataBindingPropertyId
	{
		/// <summary>
		/// The <see cref="AppointmentsDataBinding.StartDateTimeMember"/> property.
		/// </summary>
		StartDateTimeMember				= 0,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.EndDateTimeMember"/> property.
		/// </summary>
		EndDateTimeMember				= 1,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.SubjectMember"/> property.
		/// </summary>
		SubjectMember					= 2,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.DescriptionMember"/> property.
		/// </summary>
		DescriptionMember				= 3,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.AllDayEventMember"/> property.
		/// </summary>
		AllDayEventMember				= 4,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.ReminderEnabledMember"/> property.
		/// </summary>
		ReminderEnabledMember			= 5,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.ReminderIntervalMember"/> property.
		/// </summary>
		ReminderIntervalMember			= 6,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.ReminderUnitsMember"/> property.
		/// </summary>
		ReminderUnitsMember				= 7,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.OwnerKeyMember"/> property.
		/// </summary>
		OwnerKeyMember					= 8,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.DataKeyMember"/> property.
		/// </summary>
		DataKeyMember					= 9,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.RecurrenceMember"/> property.
		/// </summary>
		RecurrenceMember				= 10,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.RecurrenceIdMember"/> property.
		/// </summary>
		RecurrenceIdMember				= 11,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.OriginalStartDateTimeMember"/> property.
		/// </summary>
		OriginalStartDateTimeMember		= 12,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.AllPropertiesMember"/> property.
		/// </summary>
		AllPropertiesMember				= 13,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.DateTimeFormatString"/> property.
		/// </summary>
		DateTimeFormatString			= 14,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.DateTimeFormatInfo"/> property.
		/// </summary>
		DateTimeFormatInfo				= 15,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.AllPropertiesMemberSerializationMode"/> property.
		/// </summary>
		AllPropertiesMemberSerializationMode  = 16,

		/// <summary>
		/// The <see cref="AppointmentsDataBinding.DateMembersSerializationMode"/> property.
		/// </summary>
		DateMembersSerializationMode = 17,
	}
    #endregion AppointmentsDataBindingPropertyId enumeration

	//	BF 6.16.04	NAS2004 Vol3 - Notes Data Binding
	#region NotesDataBindingPropertyId enumeration
	/// <summary>
	/// Values that uniquely identify each data binding related property of the <see cref="Note"/> object.
	/// </summary>
	public enum NotesDataBindingPropertyId
	{
		/// <summary>
		/// The <see cref="NotesDataBinding.AllPropertiesMember"/> property.
		/// </summary>
		AllPropertiesMember				= 0,

		/// <summary>
		/// The <see cref="NotesDataBinding.DateMember"/> property.
		/// </summary>
		DateMember				= 1,

		/// <summary>
		/// The <see cref="NotesDataBinding.DescriptionMember"/> property.
		/// </summary>
		DescriptionMember				= 2,

		/// <summary>
		/// The <see cref="NotesDataBinding.OwnerKeyMember"/> property.
		/// </summary>
		OwnerKeyMember				= 3,

		/// <summary>
		/// The <see cref="NotesDataBinding.DataKeyMember"/> property.
		/// </summary>
		DataKeyMember				= 4,

		/// <summary>
		/// The <see cref="NotesDataBinding.DateTimeFormatString"/> property.
		/// </summary>
		DateTimeFormatString			= 5,

		/// <summary>
		/// The <see cref="NotesDataBinding.DateTimeFormatInfo"/> property.
		/// </summary>
		DateTimeFormatInfo				= 6,

		/// <summary>
		/// The <see cref="NotesDataBinding.AllPropertiesMemberSerializationMode"/> property.
		/// </summary>
		AllPropertiesMemberSerializationMode  = 7,

		/// <summary>
		/// The <see cref="NotesDataBinding.DateMembersSerializationMode"/> property.
		/// </summary>
		DateMembersSerializationMode = 8,
	}
	#endregion NotesDataBindingPropertyId enumeration

    #region OwnersDataBindingPropertyId enumeration
    //	BF 5.17.04	UWS968
	//	Made this public so we can identify the property in the OwnerDataError event

	/// <summary>
	/// Values that uniquely identify each bindable property of the <see cref="Owner"/> object.
	/// </summary>
	public enum OwnersDataBindingPropertyId
	{
		/// <summary>
		/// The <see cref="OwnersDataBinding.KeyMember"/> property.
		/// </summary>
		KeyMember				= 0,

		/// <summary>
		/// The <see cref="OwnersDataBinding.NameMember"/> property.
		/// </summary>
		NameMember				= 1,

		/// <summary>
		/// The <see cref="OwnersDataBinding.EmailAddressMember"/> property.
		/// </summary>
		EmailAddressMember		= 2,

		/// <summary>
		/// The <see cref="OwnersDataBinding.VisibleMember"/> property.
		/// </summary>
		VisibleMember			= 3,

		/// <summary>
		/// The <see cref="OwnersDataBinding.AllPropertiesMember"/> property.
		/// </summary>
		AllPropertiesMember		= 4,
	}
    #endregion OwnersDataBindingPropertyId enumeration

    #region DateSerializationMode enumeration
    /// <summary>
	/// Specifies whether dates will be serialized to the data source as local or UTC (Coordinated Universal Time) times.
	/// </summary>
    /// <seealso cref="DateDeserializationMode"/>
    /// <seealso cref="ActivitiesDataBindingBase.DateMembersDeserializationMode"/>
	public enum DateSerializationMode
	{
		/// <summary>
		/// Dates are serialized as local time; no conversion is performed when deserialized.
		/// </summary>
		SerializeDatesAsLocalTime = 0,

		/// <summary>
		/// Dates are serialized in UTC (Coordinated Universal Time) format; dates are converted to local time when deserialized.
		/// </summary>
		SerializeDatesAsUniversalTime = 1
	}
    #endregion DateSerializationMode enumeration

    //  BF 1/24/08  BR29935
    #region DateDeserializationMode enumeration
    /// <summary>
	/// Specifies how dates are to be de-serialized.
	/// </summary>
    /// <seealso cref="DateDeserializationMode"/>
    /// <seealso cref="ActivitiesDataBindingBase.DateMembersSerializationMode"/>
	public enum DateDeserializationMode
	{
        /// <summary>
        /// The actual deserialization mode is determined solely by the value of the <see cref="ActivitiesDataBindingBase.DateMembersSerializationMode"/> property.
        /// </summary>
        Default,

        /// <summary>
        /// The <see cref="System.DateTime.SpecifyKind"/> method is called on the deserialized date
        /// to remove a potentially inaccurate value from its <see cref="System.DateTime.Kind"/>
        /// property.
        /// </summary>
        /// <remarks>
        /// <p class="body">
        /// Because of a confirmed bug in the <see cref="System.Runtime.Serialization.Formatters.Soap.SoapFormatter"/> class
        /// whereby a DateTime's status as either a local or universal time is not preserved during the serialization process,
        /// the <see cref="ActivitiesDataBindingBase.DateMembersSerializationMode"/> property cannot always properly honor the
        /// 'SerializeDatesAsUniversalTime' setting. Dates that were serialized in the UTC format would appear in the resulting
        /// XML as local times, and thus when deserialized would be interpreted as such. Since the WinSchedule data binding layer
        /// converts dates that were serialized as universal times into the local time for the current time zone, dates would appear
        /// incorrectly, since the <see cref="System.DateTime.ToLocalTime"/> method has no effect on a DateTIme whose Kind property
        /// returns 'Local'.
        /// </p>
        /// <p class="body">
        /// If, for example, the DateMemberSerializationMode property is set to 'SerializeDatesAsUniversalTime', and an <see cref="Appointment"/>
        /// that occurs at 8AM EST is serialized to an XML file, the WinSchedule data binding layer will convert the date to its UTC equivalent,
        /// resulting in a time of 1PM. The resulting string in the XML file, however, appears with the time zone-specific "-05:00" suffix, which
        /// causes the DateTime parser to interpret the date as being expressed in local time, which in turn causes the date to incorrectly appear
        /// as 1PM EST when it is deserialized from the XML file.
        /// </p>
        /// <p class="body">
        /// By setting the <see cref="ActivitiesDataBindingBase.DateMembersDeserializationMode"/> property to 'IgnoreDateTimeKind', the WinSchedule
        /// data binding layer is instructed to remove the DateTimeKind information from the deserialized date, effectively clearing its status as a
        /// local time, so that the DateTime's ToLocalTime method performs as expected.
        /// </p>
        /// <p class="note">
        /// Note that the <see cref="ActivitiesDataBindingBase.DateMembersSerializationMode"/> exists as a convenience to the end developer to provide
        /// a solution to the specific problem created by the above mentioned limitation in the SoapFormatter class, and should only be used in scenarios
        /// where it is known conclusively that the DateTime data being deserialized contains corretc date and time information, but an incorrect or undefined
        /// value for the Kind property.
        /// </p>
        /// </remarks>
        IgnoreDateTimeKind,
	}
    #endregion DateSerializationMode enumeration

}
